import os
import os.path
import time
import librepo
import sys
from lettuce import *

CACHE = "fastestmirror.cache"

def remove_fn(fn):
    if os.path.exists(fn):
        os.remove(fn)

@before.each_feature
def feature_setup(feature):
    world.handle = None
    world.handle_options = {}
    world.packages = []
    world.filenames = []

@step(r'(\w+): (\S+)')
def handle_set_str(step, option, val):
    world.handle_options[option] = val

@step(r'Want to download location: (\S+)')
def package_to_download(step, path):
    world.filenames.append(os.path.basename(path))
    world.packages.append(path)

@step('I download the package three times')
def download_package(step):
    remove_fn(CACHE)
    for filename in world.filenames:
        remove_fn(filename)

    h = librepo.Handle()
    h.repotype = librepo.YUMREPO
    for opt, val in world.handle_options.iteritems():
        setattr(h, opt, val)

    # Download the package for the first time
    packages = []
    for package in world.packages:
        packages.append(librepo.PackageTarget(package, handle=h))
    t = time.time()
    librepo.download_packages(packages, failfast=True)
    world.t1 = time.time() - t
    for package in packages:
        assert (package.err is None)

    assert not os.path.exists(CACHE)
    for filename in world.filenames:
        assert os.path.exists(filename)
        remove_fn(filename)

    # Download the package for the second time
    h.fastestmirror = True
    h.fastestmirrorcache = CACHE
    packages = []
    for package in world.packages:
        packages.append(librepo.PackageTarget(package, handle=h))
    t = time.time()
    librepo.download_packages(packages)
    world.t2 = time.time() - t
    for package in packages:
        assert (package.err is None)

    assert os.path.exists(CACHE)
    for filename in world.filenames:
        assert os.path.exists(filename)
        remove_fn(filename)

    # Third download of the package
    h.fastestmirror = True
    h.fastestmirrorcache = CACHE
    packages = []
    for package in world.packages:
        packages.append(librepo.PackageTarget(package, handle=h))
    t = time.time()
    librepo.download_packages(packages)
    world.t3 = time.time() - t
    for package in packages:
        assert (package.err is None)

    assert os.path.exists(CACHE)
    for filename in world.filenames:
        assert os.path.exists(filename)
        remove_fn(filename)

@step('Third download with fastestmirrorcache should be the fastest')
def check_results(step):
    assert world.t2 > world.t3, "Download without cache is faster"




#!/usr/bin/python

"""
librepo - example of usage
"""

import os
import sys
import librepo
import pprint

DESTDIR = "downloaded_metadata"

if __name__ == "__main__":
    h = librepo.Handle()
    r = librepo.Result()

    # Correct repotype is important. Without repotype
    # metalink parser doesn't know suffix which should
    # be stripped off from the mirrors urls.
    h.setopt(librepo.LRO_REPOTYPE, librepo.LR_YUMREPO)

    # Set local mirrorlist file as mirrorlist
    if os.path.isfile(os.path.join(DESTDIR, "mirrorlist")):
        h.mirrorlist = os.path.join(DESTDIR, "mirrorlist")
    elif os.path.isfile(os.path.join(DESTDIR, "metalink.xml")):
        h.mirrorlist = os.path.join(DESTDIR, "metalink.xml")
    else:
        print "No mirrorlist of downloaded repodata available"
        sys.exit(0)

    # Download only the mirrorlist during perform() call.
    h.setopt(librepo.LRO_FETCHMIRRORS, True)

    h.perform(r)

    print "Urls in mirrorlist:"
    print h.mirrors
    print "Metalink file content:"
    pprint.pprint(h.metalink)



#!/usr/bin/env python

"""
librepo - Example of download_packages() function with use of end,
          failure and mirrorfailure callbacks.
"""

import librepo

if __name__ == "__main__":

    # Setup logging
    #def debug_function(msg, _):
    #    print "DEBUG: %s" % msg
    #librepo.set_debug_log_handler(debug_function)

    # Prepare handle
    h = librepo.Handle()
    h.urls = ["http://beaker-project.org/yum/client-testing/Fedora19/"]
    h.repotype = librepo.YUMREPO

    # Callbacks
    def endcb(data, status, msg):
        print "EndCb: Download of %s finished with status:" % data,
        if status == librepo.TRANSFER_SUCCESSFUL:
            print "Successfully"
        elif status == librepo.TRANSFER_ALREADYEXISTS:
            print "Already exists"
        else:
            print "Error: %s" % msg


    def mirrorfailurecb(data, msg, url):
        print "MirrorFailureCb: Download of %s from %s failed with: %s" % \
              (data, url, msg)

    # Prepare list of targets
    packages = []

    target = librepo.PackageTarget("beaker-0.14.0-1.fc18.src.rpm",
                                   handle=h,
                                   checksum_type=librepo.SHA256,
                                   checksum="e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
                                   resume=True,
                                   cbdata="beaker-0.14.0-1.fc18.src.rpm",
                                   endcb=endcb,
                                   mirrorfailurecb=mirrorfailurecb)
    packages.append(target)

    target = librepo.PackageTarget("beaker-0.13.2-1.fc17.noarch.rpm",
                                   handle=h,
                                   checksum_type=librepo.SHA256,
                                   checksum="foobar",
                                   cbdata="beaker-0.13.2-1.fc17.noarch.rpm (bad checksum)",
                                   endcb=endcb,
                                   mirrorfailurecb=mirrorfailurecb)
    packages.append(target)

    target = librepo.PackageTarget("beaker-0.13.2-1.fc17.src.rpm",
                                   handle=h,
                                   checksum_type=librepo.SHA256,
                                   checksum="xyz",
                                   cbdata="beaker-0.13.2-1.fc17.src.rpm (bad checksum)",
                                   endcb=endcb,
                                   mirrorfailurecb=mirrorfailurecb)
    packages.append(target)

    target = librepo.PackageTarget("beaker-client-0.14.1-1.fc18.noarch.rpm",
                                   handle=h,
                                   expectedsize=333333333333333,
                                   cbdata="beaker-client-0.14.1-1.fc18.noarch.rpm (bad size)",
                                   endcb=endcb,
                                   mirrorfailurecb=mirrorfailurecb)
    packages.append(target)

    target = librepo.PackageTarget("rhts-4.56-1.fc17.src.rpm_bad_filename",
                                   handle=h,
                                   cbdata="rhts-4.56-1.fc17.src.rpm_bad_filename (bad path)",
                                   endcb=endcb,
                                   mirrorfailurecb=mirrorfailurecb)
    packages.append(target)

    librepo.download_packages(packages, failfast=False)

    for target in packages:
        print "### %s: %s" % (target.local_path, target.err or "OK")
        print "Relative URL:      ", target.relative_url
        print "Destination:       ", target.dest
        print "Base URL:          ", target.base_url
        print "Checksum type:     ", target.checksum_type
        print "Expected checksum: ", target.checksum
        print "Resume:            ", bool(target.resume)
        print "Local path:        ", target.local_path
        print "Error:             ", target.err
        print




#!/usr/bin/env python

"""
librepo - download a package
"""

import os
import sys
import shutil
from pprint import pprint

import librepo

DESTDIR = "downloaded_metadata"
PROGRESSBAR_LEN = 40
finished = False

def callback(data, total_to_download, downloaded):
    """Progress callback"""
    global finished

    if total_to_download != downloaded:
        finished = False

    if total_to_download <= 0 or finished == True:
        return

    completed = int(downloaded / (total_to_download / PROGRESSBAR_LEN))
    print "%30s: [%s%s] %8s/%8s\r" % (data, '#'*completed, '-'*(PROGRESSBAR_LEN-completed), int(downloaded), int(total_to_download)),
    sys.stdout.flush()

    if total_to_download == downloaded and not finished:
        print
        finished = True
        return

if __name__ == "__main__":

    pkgs = [
        ("ImageMagick-djvu", "Packages/i/ImageMagick-djvu-6.7.5.6-3.fc17.i686.rpm"),
        ("i2c-tools-eepromer", "Packages/i/i2c-tools-eepromer-3.1.0-1.fc17.i686.rpm")
    ]

    h = librepo.Handle()
    h.setopt(librepo.LRO_URLS, ["http://ftp.linux.ncsu.edu/pub/fedora/linux/releases/17/Everything/i386/os/"])
    h.setopt(librepo.LRO_REPOTYPE, librepo.LR_YUMREPO)
    h.setopt(librepo.LRO_PROGRESSCB, callback)
    h.setopt(librepo.LRO_PROGRESSDATA, "")

    for pkg_name, pkg_url in pkgs:
        h.progressdata = pkg_name
        h.download(pkg_url)




import unittest
import librepo
from librepo import LibrepoException

class TestCaseResult(unittest.TestCase):

    def test_result_getinfo(self):
        r = librepo.Result()
        self.assertTrue(r)

        self.assertRaises(ValueError, r.getinfo, 99999999)
        self.assertFalse(r.getinfo(librepo.LRR_YUM_REPO))
        self.assertFalse(r.getinfo(librepo.LRR_YUM_REPOMD))
        self.assertRaises(LibrepoException, r.getinfo, librepo.LRR_YUM_TIMESTAMP)

        self.assertFalse(r.getinfo(librepo.LRR_RPMMD_REPO))
        self.assertFalse(r.getinfo(librepo.LRR_RPMMD_REPOMD))
        self.assertRaises(LibrepoException, r.getinfo, librepo.LRR_RPMMD_TIMESTAMP)

    def test_result_attrs(self):
        r = librepo.Result()
        self.assertTrue(r)

        self.assertRaises(AttributeError, getattr, r, 'foobar_attr')

        # Attrs should not be filled (that's why None or
        # LibrepoException is expected), but they definitelly
        # should exists (not AttributeError should be raised)
        self.assertFalse(r.yum_repo)
        self.assertFalse(r.yum_repomd)
        self.assertRaises(LibrepoException, getattr, r, 'yum_timestamp')

        self.assertFalse(r.rpmmd_repo)
        self.assertFalse(r.rpmmd_repomd)



from flask import Flask
from optparse import OptionParser
try:
    from yum_mock.yum_mock import yum_mock
except (ValueError, ImportError):
    from .yum_mock.yum_mock import yum_mock

app = Flask(__name__)
#app.register_blueprint(working_repo)
app.register_blueprint(yum_mock, url_prefix='/yum')


if __name__ == '__main__':
    parser = OptionParser("%prog [options]")
    parser.add_option(
        "-d", "--debug",
        action="store_true",
    )
    parser.add_option(
        "-p", "--port",
        default=5000,
        type="int",
    )
    parser.add_option(
        "-n", "--host",
        default="127.0.0.1",
    )
    parser.add_option(
        "--passthrough_errors",
        action="store_true",
    )
    options, args = parser.parse_args()

    kwargs = {
        "threaded": True,
        "debug": options.debug,
        "port": options.port,
        "host": options.host,
        "passthrough_errors": options.passthrough_errors,
    }

    app.run(**kwargs)



import unittest
import librepo

class TestCaseSanity(unittest.TestCase):
    def test_raw_handle_sanity(self):
        h = librepo.Handle()
        self.assertEqual(h.update, False)
        self.assertEqual(h.urls, [])
        self.assertEqual(h.mirrorlist, None)
        self.assertEqual(h.local, False)
        self.assertEqual(h.progresscb, None)
        self.assertEqual(h.progressdata, None)
        self.assertEqual(h.destdir, None)
        self.assertEqual(h.repotype, 0)
        self.assertEqual(h.useragent, None)
        self.assertEqual(h.yumdlist, None)
        self.assertEqual(h.rpmmddlist, None)
        self.assertEqual(h.yumblist, None)
        self.assertEqual(h.rpmmdblist, None)
        self.assertEqual(h.fetchmirrors, False)
        self.assertEqual(h.maxmirrortries, 0)
        self.assertEqual(h.varsub, None)
        self.assertEqual(h.mirrors, [])
        self.assertEqual(h.metalink, None)
        self.assertEqual(h.hmfcb, None)

    def test_raw_result_sanity(self):
        r = librepo.Result()
        self.assertEqual(r.yum_repo, None)
        self.assertEqual(r.yum_repomd, None)
        self.assertEqual(r.rpmmd_repo, None)
        self.assertEqual(r.rpmmd_repomd, None)

    def test_raw_result_sanity(self):
        t = librepo.PackageTarget("foo")
        self.assertEqual(t.handle, None)
        self.assertEqual(t.relative_url, "foo")
        self.assertEqual(t.dest, None)
        self.assertEqual(t.base_url, None)
        self.assertEqual(t.checksum_type, librepo.CHECKSUM_UNKNOWN)
        self.assertEqual(t.checksum, None)
        self.assertEqual(t.progresscb, None)
        self.assertEqual(t.cbdata, None)
        self.assertEqual(t.local_path, None)
        self.assertEqual(t.err, None)



import os
from flask import Blueprint, render_template, abort, send_file, request, Response
from flask import current_app
from functools import wraps

from .config import AUTH_USER, AUTH_PASS

yum_mock = Blueprint('yum_mock', __name__,
                        template_folder='templates',
                        static_folder='static')

@yum_mock.route('/static/mirrorlist/<path:path>')
def serve_mirrorlists_with_right_port(path):
    try:
        with yum_mock.open_resource('static/mirrorlist/'+path) as f:
            data = f.read()
            data = data.decode('utf-8')
            data = data.replace(":{PORT_PLACEHOLDER}", ":%d" % current_app._librepo_port)
            return data
    except IOError:
        # File probably doesn't exist or we can't read it
        abort(404)

@yum_mock.route('/static/metalink/<path:path>')
def serve_metalinks_with_right_port(path):
    try:
        with yum_mock.open_resource('static/metalink/'+path) as f:
            data = f.read()
            data = data.decode('utf-8')
            data = data.replace(":{PORT_PLACEHOLDER}", ":%d" % current_app._librepo_port)
            return data
    except IOError:
        # File probably doesn't exist or we can't read it
        abort(404)

@yum_mock.route('/harm_checksum/<keyword>/<path:path>')
def harm_checksum(keyword, path):
    """Append two newlines to content of a file (from the static dir) with
    specified keyword in the filename. If the filename doesn't contain
    the keyword, content of the file is returnen unchanged."""

    if "static/" not in path:
        # Support changing only files from static directory
        abort(400)
    path = path[path.find("static/"):]

    try:
        with yum_mock.open_resource(path) as f:
            data = f.read()
            if keyword in os.path.basename(path):
                return "%s\n\n" %data
            return data
    except IOError:
        # File probably doesn't exist or we can't read it
        abort(404)

@yum_mock.route("/not_found/<keyword>/<path:path>")
def not_found(keyword, path):
    """For each file containing keyword in the filename, http status
    code 404 will be returned"""

    if "static/" not in path:
        abort(400)
    path = path[path.find("static/"):]

    try:
        with yum_mock.open_resource(path) as f:
            data = f.read()
            if keyword in os.path.basename(path):
                abort(404)
            return data
    except IOError:
        # File probably doesn't exist or we can't read it
        abort(404)

@yum_mock.route("/badurl/<path:path>")
def badurl(path):
    """Just return 404 for each url with this prefix"""
    abort(404)

@yum_mock.route("/badgpg/<path:path>")
def badgpg(path):
    """Instead of <path>/repomd.xml.asc returns
    content of <path>/repomd.xml.asc.bad"""
    if "static/" not in path:
        abort(400)
    path = path[path.find("static/"):]
    if path.endswith("repomd.xml.asc"):
        path = path + ".bad"

    try:
        with yum_mock.open_resource(path) as f:
            return f.read()
    except IOError:
        # File probably doesn't exist or we can't read it
        abort(404)

# Basic Auth

def check_auth(username, password):
    """This function is called to check if a username /
    password combination is valid.
    """
    return username == AUTH_USER and password == AUTH_PASS

def authenticate():
    """Sends a 401 response that enables basic auth"""
    return Response(
    'Could not verify your access level for that URL.\n'
    'You have to login with proper credentials', 401,
    {'WWW-Authenticate': 'Basic realm="Login Required"'})

def requires_auth(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        auth = request.authorization
        if not auth or not check_auth(auth.username, auth.password):
            return authenticate()
        return f(*args, **kwargs)
    return decorated

@yum_mock.route("/auth_basic/<path:path>")
@requires_auth
def secret_repo_basic_auth(path):
    """Page secured with basic HTTP auth
    User: admin Password: secret"""
    if "static/" not in path:
        abort(400)
    path = path[path.find("static/"):]

    try:
        with yum_mock.open_resource(path) as f:
            data = f.read()
            return data
    except IOError:
        abort(404)




import sys
import time
import shutil
import os.path
import tempfile
import unittest

import librepo

from tests.base import Context, TestCaseWithFlask, TEST_DATA
from tests.servermock.server import app
import tests.servermock.yum_mock.config as config

PUB_KEY = TEST_DATA+"/key.pub"

class TestCaseYumRepoDownloading(TestCaseWithFlask):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp(prefix="librepotest-")
        # Import public key into the temporary gpg keyring
        self._gnupghome = os.environ.get('GNUPGHOME')
        gpghome = os.path.join(self.tmpdir, "keyring")
        os.mkdir(gpghome, 0o700)
        os.environ['GNUPGHOME'] = gpghome
        self.ctx = Context()
        self.ctx.op_import(open(PUB_KEY, 'rb'))

    def tearDown(self):
        if self._gnupghome is None:
            os.environ.pop('GNUPGHOME')
        else:
            os.environ['GNUPGHOME'] = self._gnupghome
        shutil.rmtree(self.tmpdir, True)

    def test_download_repo_01(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)
        timestamp  = r.getinfo(librepo.LRR_YUM_TIMESTAMP)

        self.assertEqual(yum_repo,
            { #'deltainfo': None,
              'destdir': self.tmpdir,
              'filelists': self.tmpdir+'/repodata/aeca08fccd3c1ab831e1df1a62711a44ba1922c9-filelists.xml.gz',
              'filelists_db': self.tmpdir+'/repodata/4034dcea76c94d3f7a9616779539a4ea8cac288f-filelists.sqlite.bz2',
              #'group': None,
              #'group_gz': None,
              #'origin': None,
              'other': self.tmpdir+'/repodata/a8977cdaa0b14321d9acfab81ce8a85e869eee32-other.xml.gz',
              'other_db': self.tmpdir+'/repodata/fd96942c919628895187778633001cff61e872b8-other.sqlite.bz2',
              #'prestodelta': None,
              'primary': self.tmpdir+'/repodata/4543ad62e4d86337cd1949346f9aec976b847b58-primary.xml.gz',
              'primary_db': self.tmpdir+'/repodata/735cd6294df08bdf28e2ba113915ca05a151118e-primary.sqlite.bz2',
              'repomd': self.tmpdir+'/repodata/repomd.xml',
              #'updateinfo': None,
              'url': url,
              'signature': None,
              'mirrorlist': None,
              'metalink': None}
        )

        self.maxDiff = None
        self.assertEqual(yum_repomd,
            {   'content_tags': [],
                #'deltainfo': None,
                'distro_tags': [],
                'filelists': {
                    'checksum': 'aeca08fccd3c1ab831e1df1a62711a44ba1922c9',
                    'header_checksum': None,
                    'header_checksum_type': None,
                    'checksum_open': '52d30ae3162ca863c63c345ffdb7f0e10c1414a5',
                    'checksum_open_type': 'sha1',
                    'checksum_type': 'sha1',
                    'db_version': 0,
                    'location_href': 'repodata/aeca08fccd3c1ab831e1df1a62711a44ba1922c9-filelists.xml.gz',
                    'size': 43310,
                    'size_open': 735088,
                    'timestamp': 1347459930},
                'filelists_db': {
                    'checksum': '4034dcea76c94d3f7a9616779539a4ea8cac288f',
                    'header_checksum': None,
                    'header_checksum_type': None,
                    'checksum_open': '949c6b7b605b2bc66852630c841a5003603ca5b2',
                    'checksum_open_type': 'sha1',
                    'checksum_type': 'sha1',
                    'db_version': 10,
                    'location_href': 'repodata/4034dcea76c94d3f7a9616779539a4ea8cac288f-filelists.sqlite.bz2',
                    'size': 22575,
                    'size_open': 201728,
                    'timestamp': 1347459931},
                #'group': None,
                #'group_gz': None,
                #'origin': None,
                'other': {
                    'checksum': 'a8977cdaa0b14321d9acfab81ce8a85e869eee32',
                    'header_checksum': None,
                    'header_checksum_type': None,
                    'checksum_open': '4b5b8874fb233a626b03b3260a1aa08dce90e81a',
                    'checksum_open_type': 'sha1',
                    'checksum_type': 'sha1',
                    'db_version': 0,
                    'location_href': 'repodata/a8977cdaa0b14321d9acfab81ce8a85e869eee32-other.xml.gz',
                    'size': 807,
                    'size_open': 1910,
                    'timestamp': 1347459930},
                'other_db': {
                    'checksum': 'fd96942c919628895187778633001cff61e872b8',
                    'header_checksum': None,
                    'header_checksum_type': None,
                    'checksum_open': 'c5262f62b6b3360722b9b2fb5d0a9335d0a51112',
                    'checksum_open_type': 'sha1',
                    'checksum_type': 'sha1',
                    'db_version': 10,
                    'location_href': 'repodata/fd96942c919628895187778633001cff61e872b8-other.sqlite.bz2',
                    'size': 1407,
                    'size_open': 8192,
                    'timestamp': 1347459931},
                #'prestodelta': None,
                'primary': {
                    'checksum': '4543ad62e4d86337cd1949346f9aec976b847b58',
                    'header_checksum': None,
                    'header_checksum_type': None,
                    'checksum_open': '68457ceb8e20bda004d46e0a4dfa4a69ce71db48',
                    'checksum_open_type': 'sha1',
                    'checksum_type': 'sha1',
                    'db_version': 0,
                    'location_href': 'repodata/4543ad62e4d86337cd1949346f9aec976b847b58-primary.xml.gz',
                    'size': 936,
                    'size_open': 3385,
                    'timestamp': 1347459930},
                'primary_db': {
                    'checksum': '735cd6294df08bdf28e2ba113915ca05a151118e',
                    'header_checksum': None,
                    'header_checksum_type': None,
                    'checksum_open': 'ba636386312e1b597fc4feb182d04c059b2a77d5',
                    'checksum_open_type': 'sha1',
                    'checksum_type': 'sha1',
                    'db_version': 10,
                    'location_href': 'repodata/735cd6294df08bdf28e2ba113915ca05a151118e-primary.sqlite.bz2',
                    'size': 2603,
                    'size_open': 23552,
                    'timestamp': 1347459931},
                'repo_tags': [],
                'revision': '1347459931',
                #'updateinfo': None
                }
        )

        self.assertEqual(timestamp, 1347459931)

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

        self.assertFalse(h.mirrors)
        self.assertFalse(h.metalink)

    def test_download_repo_01_rpmmd(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_RPMMDREPO
        h.destdir = self.tmpdir
        h.perform(r)

        rpmmd_repo   = r.getinfo(librepo.LRR_RPMMD_REPO)
        rpmmd_repomd = r.getinfo(librepo.LRR_RPMMD_REPOMD)
        timestamp  = r.getinfo(librepo.LRR_RPMMD_TIMESTAMP)

        self.assertEqual(rpmmd_repo,
            { 'destdir': self.tmpdir,
              'url': url,
              'signature': None,
              'mirrorlist': None,
              'metalink': None,
              'repomd': self.tmpdir+'/repodata/repomd.xml',
              'paths': {
                  'primary': self.tmpdir+'/repodata/4543ad62e4d86337cd1949346f9aec976b847b58-primary.xml.gz',
                  'primary_db': self.tmpdir+'/repodata/735cd6294df08bdf28e2ba113915ca05a151118e-primary.sqlite.bz2',
                  'filelists': self.tmpdir+'/repodata/aeca08fccd3c1ab831e1df1a62711a44ba1922c9-filelists.xml.gz',
                  'filelists_db': self.tmpdir+'/repodata/4034dcea76c94d3f7a9616779539a4ea8cac288f-filelists.sqlite.bz2',
                  'other': self.tmpdir+'/repodata/a8977cdaa0b14321d9acfab81ce8a85e869eee32-other.xml.gz',
                  'other_db': self.tmpdir+'/repodata/fd96942c919628895187778633001cff61e872b8-other.sqlite.bz2',
                  }
            }
        )

        self.assertEqual(rpmmd_repomd,
            {   'revision': '1347459931',
                'content_tags': [],
                'distro_tags': [],
                'repo_tags': [],
                'records': {
                    'primary': {
                        'checksum': '4543ad62e4d86337cd1949346f9aec976b847b58',
                        'header_checksum': None,
                        'header_checksum_type': None,
                        'checksum_open': '68457ceb8e20bda004d46e0a4dfa4a69ce71db48',
                        'checksum_open_type': 'sha1',
                        'checksum_type': 'sha1',
                        'db_version': 0,
                        'location_href': 'repodata/4543ad62e4d86337cd1949346f9aec976b847b58-primary.xml.gz',
                        'size': 936,
                        'size_open': 3385,
                        'timestamp': 1347459930},
                    'primary_db': {
                        'checksum': '735cd6294df08bdf28e2ba113915ca05a151118e',
                        'header_checksum': None,
                        'header_checksum_type': None,
                        'checksum_open': 'ba636386312e1b597fc4feb182d04c059b2a77d5',
                        'checksum_open_type': 'sha1',
                        'checksum_type': 'sha1',
                        'db_version': 10,
                        'location_href': 'repodata/735cd6294df08bdf28e2ba113915ca05a151118e-primary.sqlite.bz2',
                        'size': 2603,
                        'size_open': 23552,
                        'timestamp': 1347459931},
                    'filelists': {
                        'checksum': 'aeca08fccd3c1ab831e1df1a62711a44ba1922c9',
                        'header_checksum': None,
                        'header_checksum_type': None,
                        'checksum_open': '52d30ae3162ca863c63c345ffdb7f0e10c1414a5',
                        'checksum_open_type': 'sha1',
                        'checksum_type': 'sha1',
                        'db_version': 0,
                        'location_href': 'repodata/aeca08fccd3c1ab831e1df1a62711a44ba1922c9-filelists.xml.gz',
                        'size': 43310,
                        'size_open': 735088,
                        'timestamp': 1347459930},
                    'filelists_db': {
                        'checksum': '4034dcea76c94d3f7a9616779539a4ea8cac288f',
                        'header_checksum': None,
                        'header_checksum_type': None,
                        'checksum_open': '949c6b7b605b2bc66852630c841a5003603ca5b2',
                        'checksum_open_type': 'sha1',
                        'checksum_type': 'sha1',
                        'db_version': 10,
                        'location_href': 'repodata/4034dcea76c94d3f7a9616779539a4ea8cac288f-filelists.sqlite.bz2',
                        'size': 22575,
                        'size_open': 201728,
                        'timestamp': 1347459931},
                    'other': {
                        'checksum': 'a8977cdaa0b14321d9acfab81ce8a85e869eee32',
                        'header_checksum': None,
                        'header_checksum_type': None,
                        'checksum_open': '4b5b8874fb233a626b03b3260a1aa08dce90e81a',
                        'checksum_open_type': 'sha1',
                        'checksum_type': 'sha1',
                        'db_version': 0,
                        'location_href': 'repodata/a8977cdaa0b14321d9acfab81ce8a85e869eee32-other.xml.gz',
                        'size': 807,
                        'size_open': 1910,
                        'timestamp': 1347459930},
                    'other_db': {
                        'checksum': 'fd96942c919628895187778633001cff61e872b8',
                        'header_checksum': None,
                        'header_checksum_type': None,
                        'checksum_open': 'c5262f62b6b3360722b9b2fb5d0a9335d0a51112',
                        'checksum_open_type': 'sha1',
                        'checksum_type': 'sha1',
                        'db_version': 10,
                        'location_href': 'repodata/fd96942c919628895187778633001cff61e872b8-other.sqlite.bz2',
                        'size': 1407,
                        'size_open': 8192,
                        'timestamp': 1347459931},
                    }
                }
        )

        self.assertEqual(timestamp, 1347459931)

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(rpmmd_repo["destdir"]))
        for key in rpmmd_repo.get("records", []):
            self.assertTrue(os.path.isfile(rpmmd_repo[key]))

        self.assertFalse(h.mirrors)
        self.assertFalse(h.metalink)



    def test_download_repo_02(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_02_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertEqual(yum_repo,
            {'deltainfo': self.tmpdir+'/repodata/32d3307b672abf7356061912fa3dc9b54071c03a75c671111c1c8daf5ed1eb7e-deltainfo.xml.gz',
             'destdir': self.tmpdir,
             'filelists': self.tmpdir+'/repodata/2431efa18b5de6bfddb87da2a526362108226752d46ef3a298cd4bf39ba16b1d-filelists.xml.gz',
             'filelists_db': self.tmpdir+'/repodata/5b37f89f9f4474801ec5f23dc30d3d6cf9cf663cb75a6656aaa864a041836ffe-filelists.sqlite.bz2',
             'group': self.tmpdir+'/repodata/5b3b362d644e8fa3b359db57be0ff5de8a08365ce9a59cddc3205244a968231e-comps.xml',
             'group_gz': self.tmpdir+'/repodata/c395ae7d8a9117f4e81aa23e37fb9da9865b50917f5f701b50d422875bb0cb14-comps.xml.gz',
             'origin': self.tmpdir+'/repodata/c949d2b2371fab1a03d03b41057004caf1133a56e4c9236f63b3163ad358c941-pkgorigins.gz',
             'other': self.tmpdir+'/repodata/76b2cfb04531a66e382f187e6a7c90905940d2b2f315b7fd738b839887d83c35-other.xml.gz',
             'other_db': self.tmpdir+'/repodata/705a58b0e169bf1d2ade8e4aacc515086644ce16cee971906f920c798c5b17d0-other.sqlite.bz2',
             'prestodelta': self.tmpdir+'/repodata/26e351e1a38eb1524574e86ab130ea4db780aa1a4a8bb741d37595ed203f931c-prestodelta.xml.gz',
             'primary': self.tmpdir+'/repodata/5a8e6bbb940b151103b3970a26e32b8965da9e90a798b1b80ee4325308149d8d-primary.xml.gz',
             'primary_db': self.tmpdir+'/repodata/a09c42730c03b0d5defa3fd9213794c49e9bafbc67acdd8d4e87a2adf30b8752-primary.sqlite.bz2',
             'repomd': self.tmpdir+'/repodata/repomd.xml',
             'updateinfo': self.tmpdir+'/repodata/65c4f66e2808d328890505c3c2f13bb35a96f457d1c21a6346191c4dc07e6080-updateinfo.xml.gz',
             'url': url,
             'signature': None,
             'mirrorlist': None,
             'metalink': None}
        )

        self.assertEqual(yum_repomd,
            {'content_tags': ['binary-i386'],
             'deltainfo': {'checksum': '32d3307b672abf7356061912fa3dc9b54071c03a75c671111c1c8daf5ed1eb7e',
                           'header_checksum': None,
                           'header_checksum_type': None,
                           'checksum_open': '8a35a38aef926fd88f479f03a9a22e1ab7aa8bd1aeaa9d05cd696f101eee2846',
                           'checksum_open_type': 'sha256',
                           'checksum_type': 'sha256',
                           'db_version': 0,
                           'location_href': 'repodata/32d3307b672abf7356061912fa3dc9b54071c03a75c671111c1c8daf5ed1eb7e-deltainfo.xml.gz',
                           'size': 300,
                           'size_open': 492,
                           'timestamp': 1355335029},
             'distro_tags': [('cpe:/o:fedoraproject:fedora:17', 'r')],
             'filelists': {'checksum': '2431efa18b5de6bfddb87da2a526362108226752d46ef3a298cd4bf39ba16b1d',
                           'header_checksum': None,
                           'header_checksum_type': None,
                           'checksum_open': 'afa4a01d7a692ab8105a39fed5535b5011f0c68de0efbc98f9d6ffea36de85fe',
                           'checksum_open_type': 'sha256',
                           'checksum_type': 'sha256',
                           'db_version': 0,
                           'location_href': 'repodata/2431efa18b5de6bfddb87da2a526362108226752d46ef3a298cd4bf39ba16b1d-filelists.xml.gz',
                           'size': 43338,
                           'size_open': 735112,
                           'timestamp': 1355393567},
             'filelists_db': {'checksum': '5b37f89f9f4474801ec5f23dc30d3d6cf9cf663cb75a6656aaa864a041836ffe',
                              'header_checksum': None,
                              'header_checksum_type': None,
                              'checksum_open': '8239ecd9334a3bc4dfa9a242f7c4d545b08451a1ad468458e20f3d3f768652c3',
                              'checksum_open_type': 'sha256',
                              'checksum_type': 'sha256',
                              'db_version': 10,
                              'location_href': 'repodata/5b37f89f9f4474801ec5f23dc30d3d6cf9cf663cb75a6656aaa864a041836ffe-filelists.sqlite.bz2',
                              'size': 23038,
                              'size_open': 200704,
                              'timestamp': 1355393568},
             'group': {'checksum': '5b3b362d644e8fa3b359db57be0ff5de8a08365ce9a59cddc3205244a968231e',
                       'header_checksum': None,
                       'header_checksum_type': None,
                       'checksum_open': None,
                       'checksum_open_type': None,
                       'checksum_type': 'sha256',
                       'db_version': 0,
                       'location_href': 'repodata/5b3b362d644e8fa3b359db57be0ff5de8a08365ce9a59cddc3205244a968231e-comps.xml',
                       'size': 679,
                       'size_open': 0,
                       'timestamp': 1355393567},
             'group_gz': {'checksum': 'c395ae7d8a9117f4e81aa23e37fb9da9865b50917f5f701b50d422875bb0cb14',
                          'header_checksum': None,
                          'header_checksum_type': None,
                          'checksum_open': '5b3b362d644e8fa3b359db57be0ff5de8a08365ce9a59cddc3205244a968231e',
                          'checksum_open_type': 'sha256',
                          'checksum_type': 'sha256',
                          'db_version': 0,
                          'location_href': 'repodata/c395ae7d8a9117f4e81aa23e37fb9da9865b50917f5f701b50d422875bb0cb14-comps.xml.gz',
                          'size': 331,
                          'size_open': 679,
                          'timestamp': 1355393567},
             'origin': {'checksum': 'c949d2b2371fab1a03d03b41057004caf1133a56e4c9236f63b3163ad358c941',
                        'header_checksum': None,
                        'header_checksum_type': None,
                        'checksum_open': '3928c6aadcfdff101f4482db68c0d07f5777b1c7ad8424e41358bc5e87b8465b',
                        'checksum_open_type': 'sha256',
                        'checksum_type': 'sha256',
                        'db_version': 0,
                        'location_href': 'repodata/c949d2b2371fab1a03d03b41057004caf1133a56e4c9236f63b3163ad358c941-pkgorigins.gz',
                        'size': 140,
                        'size_open': 364,
                        'timestamp': 1355315696},
             'other': {'checksum': '76b2cfb04531a66e382f187e6a7c90905940d2b2f315b7fd738b839887d83c35',
                       'header_checksum': None,
                       'header_checksum_type': None,
                       'checksum_open': '2169e09e2c6c91393d38866c501a8697d0a1d698dd3b1027969dc16d291d8915',
                       'checksum_open_type': 'sha256',
                       'checksum_type': 'sha256',
                       'db_version': 0,
                       'location_href': 'repodata/76b2cfb04531a66e382f187e6a7c90905940d2b2f315b7fd738b839887d83c35-other.xml.gz',
                       'size': 826,
                       'size_open': 1934,
                       'timestamp': 1355393567},
             'other_db': {'checksum': '705a58b0e169bf1d2ade8e4aacc515086644ce16cee971906f920c798c5b17d0',
                          'header_checksum': None,
                          'header_checksum_type': None,
                          'checksum_open': '916ca5e879387dc1da51b57266bda28a2569d1773ca6c8ea80abe99d9adb373e',
                          'checksum_open_type': 'sha256',
                          'checksum_type': 'sha256',
                          'db_version': 10,
                          'location_href': 'repodata/705a58b0e169bf1d2ade8e4aacc515086644ce16cee971906f920c798c5b17d0-other.sqlite.bz2',
                          'size': 1462,
                          'size_open': 8192,
                          'timestamp': 1355393568},
             'prestodelta': {'checksum': '26e351e1a38eb1524574e86ab130ea4db780aa1a4a8bb741d37595ed203f931c',
                             'header_checksum': None,
                             'header_checksum_type': None,
                             'checksum_open': '0052b222add25fed094793c24e73aa07fd598f43f73c1643de26c5e81f6d8c07',
                             'checksum_open_type': 'sha256',
                             'checksum_type': 'sha256',
                             'db_version': 0,
                             'location_href': 'repodata/26e351e1a38eb1524574e86ab130ea4db780aa1a4a8bb741d37595ed203f931c-prestodelta.xml.gz',
                             'size': 336,
                             'size_open': 574,
                             'timestamp': 1337937059},
             'primary': {'checksum': '5a8e6bbb940b151103b3970a26e32b8965da9e90a798b1b80ee4325308149d8d',
                         'header_checksum': None,
                         'header_checksum_type': None,
                         'checksum_open': 'b8d60e74c38b94f255c08c3fe5e10c166dcb52f2c4bfec6cae097a68fdd75e74',
                         'checksum_open_type': 'sha256',
                         'checksum_type': 'sha256',
                         'db_version': 0,
                         'location_href': 'repodata/5a8e6bbb940b151103b3970a26e32b8965da9e90a798b1b80ee4325308149d8d-primary.xml.gz',
                         'size': 956,
                         'size_open': 3411,
                         'timestamp': 1355393567},
             'primary_db': {'checksum': 'a09c42730c03b0d5defa3fd9213794c49e9bafbc67acdd8d4e87a2adf30b8752',
                            'header_checksum': None,
                            'header_checksum_type': None,
                            'checksum_open': '27b2200efa2c518e5dd5a59deb9ab33c2abca74cb74f5241e612b15931dcec37',
                            'checksum_open_type': 'sha256',
                            'checksum_type': 'sha256',
                            'db_version': 10,
                            'location_href': 'repodata/a09c42730c03b0d5defa3fd9213794c49e9bafbc67acdd8d4e87a2adf30b8752-primary.sqlite.bz2',
                            'size': 2649,
                            'size_open': 23552,
                            'timestamp': 1355393568},
             'repo_tags': ['test'],
             'revision': '1355393568',
             'updateinfo': {'checksum': '65c4f66e2808d328890505c3c2f13bb35a96f457d1c21a6346191c4dc07e6080',
                            'header_checksum': None,
                            'header_checksum_type': None,
                            'checksum_open': 'ded9c95e1b88197c906603b5d9693c579cb0afeade3bc7f8ec6cae06b962477d',
                            'checksum_open_type': 'sha256',
                            'checksum_type': 'sha256',
                            'db_version': 0,
                            'location_href': 'repodata/65c4f66e2808d328890505c3c2f13bb35a96f457d1c21a6346191c4dc07e6080-updateinfo.xml.gz',
                            'size': 55,
                            'size_open': 42,
                            'timestamp': 1354188048}
             }
        )

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

        self.assertFalse(h.mirrors)
        self.assertFalse(h.metalink)

    def test_download_repo_from_bad_url(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.BADURL)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir

        raised = False
        try:
            h.perform(r)
        except librepo.LibrepoException as err:
            raised = True
            unicode_type = unicode if sys.version_info[0] < 3 else str
            self.assertTrue(isinstance(err.args[1], unicode_type))
            self.assertFalse(h.mirrors)
            self.assertFalse(h.metalink)
        finally:
            self.assertTrue(raised)

    def test_partial_download_repo_01(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.yumdlist = []
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)

        self.assertEqual(yum_repo,
            {#'deltainfo': None,
             'destdir': self.tmpdir,
             #'filelists': None,
             #'filelists_db': None,
             #'group': None,
             #'group_gz': None,
             #'origin': None,
             #'other': None,
             #'other_db': None,
             #'prestodelta': None,
             #'primary': None,
             #'primary_db': None,
             'repomd': self.tmpdir+'/repodata/repomd.xml',
             #'updateinfo': None,
             'url': url,
             'signature': None,
             'mirrorlist': None,
             'metalink': None}
        )

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

        self.assertFalse(h.mirrors)
        self.assertFalse(h.metalink)

    def test_partial_download_repo_02(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.yumdlist = ["other", "primary"]
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)

        self.assertEqual(yum_repo,
            {#'deltainfo': None,
             'destdir': self.tmpdir,
             #'filelists': None,
             #'filelists_db': None,
             #'group': None,
             #'group_gz': None,
             #'origin': None,
             'other': self.tmpdir+'/repodata/a8977cdaa0b14321d9acfab81ce8a85e869eee32-other.xml.gz',
             #'other_db': None,
             #'prestodelta': None,
             'primary': self.tmpdir+'/repodata/4543ad62e4d86337cd1949346f9aec976b847b58-primary.xml.gz',
             #'primary_db': None,
             'repomd': self.tmpdir+'/repodata/repomd.xml',
             #'updateinfo': None,
             'url': url,
             'signature': None,
             'mirrorlist': None,
             'metalink': None}
        )

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

        self.assertFalse(h.mirrors)
        self.assertFalse(h.metalink)

    def test_partial_download_repo_03(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.yumblist = ["other", "filelists"]
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)

        self.assertEqual(yum_repo,
            { #'deltainfo': None,
              'destdir': self.tmpdir,
              #'filelists': self.tmpdir+'/repodata/aeca08fccd3c1ab831e1df1a62711a44ba1922c9-filelists.xml.gz',
              'filelists_db': self.tmpdir+'/repodata/4034dcea76c94d3f7a9616779539a4ea8cac288f-filelists.sqlite.bz2',
              #'group': None,
              #'group_gz': None,
              #'origin': None,
              #'other': self.tmpdir+'/repodata/a8977cdaa0b14321d9acfab81ce8a85e869eee32-other.xml.gz',
              'other_db': self.tmpdir+'/repodata/fd96942c919628895187778633001cff61e872b8-other.sqlite.bz2',
              #'prestodelta': None,
              'primary': self.tmpdir+'/repodata/4543ad62e4d86337cd1949346f9aec976b847b58-primary.xml.gz',
              'primary_db': self.tmpdir+'/repodata/735cd6294df08bdf28e2ba113915ca05a151118e-primary.sqlite.bz2',
              'repomd': self.tmpdir+'/repodata/repomd.xml',
              #'updateinfo': None,
              'url': url,
              'signature': None,
              'mirrorlist': None,
              'metalink': None}
        )

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

        self.assertFalse(h.mirrors)
        self.assertFalse(h.metalink)

    def test_partial_download_repo_with_substitution(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.yumdlist = ["foo", "primary"]
        h.yumslist = [("foo", "other")]
        h.perform(r)

        yum_repo = r.getinfo(librepo.LRR_YUM_REPO)

        self.assertEqual(yum_repo,
            {#'deltainfo': None,
             'destdir': self.tmpdir,
             #'filelists': None,
             #'filelists_db': None,
             #'group': None,
             #'group_gz': None,
             #'origin': None,
             'other': self.tmpdir+'/repodata/a8977cdaa0b14321d9acfab81ce8a85e869eee32-other.xml.gz',
             #'other_db': None,
             #'prestodelta': None,
             'primary': self.tmpdir+'/repodata/4543ad62e4d86337cd1949346f9aec976b847b58-primary.xml.gz',
             #'primary_db': None,
             'repomd': self.tmpdir+'/repodata/repomd.xml',
             #'updateinfo': None,
             'url': url,
             'signature': None,
             'mirrorlist': None,
             'metalink': None}
        )

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

        self.assertFalse(h.mirrors)
        self.assertFalse(h.metalink)

    def test_download_repo_01_without_result_object(self):
        h = librepo.Handle()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        r = h.perform()

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)

    def test_download_repo_01_with_checksum_check(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)

    def test_download_corrupted_repo_01_with_checksum_check(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s%s" % (self.MOCKURL, config.HARMCHECKSUM % "primary.xml", config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)

    def test_download_repo_with_gpg_check(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.gpgcheck = True
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertTrue("signature" in yum_repo and yum_repo["signature"])
        self.assertTrue(self.tmpdir+'/repodata/repomd.xml.asc' == yum_repo["signature"] )
        self.assertTrue(os.path.isfile(yum_repo["signature"]))

    def test_download_repo_with_gpg_check_bad_signature(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s%s" % (self.MOCKURL, config.BADGPG, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.gpgcheck = True
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertTrue("signature" not in yum_repo or yum_repo["signature"])

    def test_download_repo_01_with_missing_file(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s%s" % (self.MOCKURL, config.MISSINGFILE % "primary.xml", config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertTrue(os.path.getsize(yum_repo["primary"]) == 0)

    def test_download_repo_01_with_missing_unwanted_file(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s%s" % (self.MOCKURL, config.MISSINGFILE % "primary.xml", config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.yumdlist = ["other"]
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)

    def test_bad_mirrorlist_url(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.BADURL)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

# Metalink tests

    def test_download_only_metalink(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_GOOD_01)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.fetchmirrors = True
        h.perform(r)
        self.assertEqual(h.mirrors, ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])

        self.assertEqual(h.metalink,
            {'timestamp': 1347459931,
             'hashes': [
                 ('md5', 'f76409f67a84bcd516131d5cc98e57e1'),
                 ('sha1', '75125e73304c21945257d9041a908d0d01d2ca16'),
                 ('sha256', 'bef5d33dc68f47adc7b31df448851b1e9e6bae27840f28700fff144881482a6a'),
                 ('sha512', 'e40060c747895562e945a68967a04d1279e4bd8507413681f83c322479aa564027fdf3962c2d875089bfcb9317d3a623465f390dc1f4acef294711168b807af0')],
             'size': 2621,
             'urls': [{
                 'url': 'http://127.0.0.1:%d/yum/static/01/repodata/repomd.xml' % self.PORT,
                 'type': 'http',
                 'protocol': 'http',
                 'location': 'CZ',
                 'preference': 100}],
             'filename': 'repomd.xml'}
            )

    def test_download_repo_01_via_metalink_with_alternates(self):
        h = librepo.Handle()
        h.metalinkurl = "%s%s" % (self.MOCKURL, config.METALINK_WITH_ALTERNATES)
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        r = h.perform()

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertEqual(yum_repo["url"], "http://127.0.0.1:%d/yum/static/01/" % self.PORT)
        self.assertTrue(yum_repomd)
        self.assertEqual(h.mirrors, ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])
        self.assertEqual(h.metalink,
            {'timestamp': 1381706941,
             'hashes': [
                 ('md5', 'bad'),
                 ('sha1', 'bad'),
                 ('sha256', 'bad'),
                 ('sha512', 'bad')],
             'size': 4761,
             'urls': [{
                 'url': 'http://127.0.0.1:%d/yum/static/01/repodata/repomd.xml' % self.PORT,
                 'type': 'http',
                 'protocol': 'http',
                 'location': 'CZ',
                 'preference': 100}],
             'filename': 'repomd.xml',
             'alternates': [{
                 'timestamp': 1347459931,
                 'hashes': [
                    ('md5', 'f76409f67a84bcd516131d5cc98e57e1'),
                    ('sha1', '75125e73304c21945257d9041a908d0d01d2ca16'),
                    ('sha256', 'bef5d33dc68f47adc7b31df448851b1e9e6bae27840f28700fff144881482a6a'),
                    ('sha512', 'e40060c747895562e945a68967a04d1279e4bd8507413681f83c322479aa564027fdf3962c2d875089bfcb9317d3a623465f390dc1f4acef294711168b807af0')
                    ],
                'size': 2621},
                {
                 'timestamp': 123,
                 'hashes': [
                    ('sha1', 'foobar'),
                    ],
                'size': 456}]
            }
            )

    def test_download_repo_01_via_metalink_01(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_GOOD_01)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertEqual(yum_repo["url"], "http://127.0.0.1:%d/yum/static/01/" % self.PORT)
        self.assertTrue(yum_repomd)

        self.assertEqual(h.mirrors, ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])
        self.assertEqual(h.metalink,
            {'timestamp': 1347459931,
             'hashes': [
                 ('md5', 'f76409f67a84bcd516131d5cc98e57e1'),
                 ('sha1', '75125e73304c21945257d9041a908d0d01d2ca16'),
                 ('sha256', 'bef5d33dc68f47adc7b31df448851b1e9e6bae27840f28700fff144881482a6a'),
                 ('sha512', 'e40060c747895562e945a68967a04d1279e4bd8507413681f83c322479aa564027fdf3962c2d875089bfcb9317d3a623465f390dc1f4acef294711168b807af0')],
             'size': 2621,
             'urls': [{
                 'url': 'http://127.0.0.1:%d/yum/static/01/repodata/repomd.xml' % self.PORT,
                 'type': 'http',
                 'protocol': 'http',
                 'location': 'CZ',
                 'preference': 100}],
             'filename': 'repomd.xml'}
            )

    def test_download_repo_01_via_metalink_02(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_GOOD_02)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)

    def test_download_repo_01_via_metalink_badfilename(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_BADFILENAME)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertEqual(yum_repo, None)
        self.assertEqual(yum_repomd, None)

    def test_download_repo_01_via_metalink_badchecksum(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_BADCHECKSUM)
        h.metalinkurl = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo["metalink"])
        # All other values shoud be None, [], {} or equivalent
        for key in yum_repo:
            if key == "metalink":
                continue
            self.assertFalse(yum_repo[key])
        for key in yum_repomd:
            self.assertFalse(yum_repomd[key])

    def test_download_repo_01_via_metalink_nourls(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_NOURLS)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertEqual(yum_repo, None)
        self.assertEqual(yum_repomd, None)

    def test_download_repo_01_via_metalink_badfirsturl(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_BADFIRSTURL)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertEqual(yum_repo["url"], "http://127.0.0.1:%d/yum/static/01/" % self.PORT)

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

    def test_download_repo_01_via_metalink_badfirsturl_maxmirrortries(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_BADFIRSTHOST)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.maxmirrortries = 1

        # Because first host is bad and maxmirrortries == 1
        # Download should fail
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

    def test_download_repo_01_via_metalink_badfirsthost_fastestmirror(self):
        time.sleep(0.5)
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_BADFIRSTHOST)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.fastestmirror = True
        h.fastestmirrortimeout = 5.0
        h.maxmirrortries = 1

        # First host is bad, but fastestmirror is used and thus
        # working mirror should be added to the first position
        # and download should be successful even if maxmirrortries
        # is equal to 1.
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertEqual(yum_repo["url"], "http://127.0.0.1:%d/yum/static/01/" % self.PORT)

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

    def test_download_repo_01_via_metalink_badfirsthost_fastestmirror_with_cache(self):
        time.sleep(0.5)
        h = librepo.Handle()
        r = librepo.Result()

        cache = os.path.join(self.tmpdir, "fastestmirror.cache")
        self.assertFalse(os.path.exists(cache))

        url = "%s%s" % (self.MOCKURL, config.METALINK_BADFIRSTHOST)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.fastestmirror = True
        h.fastestmirrortimeout = 5.0
        h.fastestmirrorcache = cache
        h.maxmirrortries = 1

        # First host is bad, but fastestmirror is used and thus
        # working mirror should be added to the first position
        # and download should be successful even if maxmirrortries
        # is equal to 1.
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)
        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertEqual(yum_repo["url"], "http://127.0.0.1:%d/yum/static/01/" % self.PORT)
        self.assertTrue(os.path.exists(cache))

        shutil.rmtree(os.path.join(self.tmpdir, "repodata"))

        # Try again, this time, fastestmirror cache should be used
        h.perform()

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)
        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertEqual(yum_repo["url"], "http://127.0.0.1:%d/yum/static/01/" % self.PORT)
        self.assertTrue(os.path.exists(cache))

    def test_download_repo_01_via_metalink_firsturlhascorruptedfiles(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_FIRSTURLHASCORRUPTEDFILES)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertEqual(yum_repo["url"],
            "http://127.0.0.1:%d/yum/harm_checksum/primary.xml/static/01/" % self.PORT)

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

    def test_download_repo_01_with_baseurl_and_metalink_specified_only_fetchmirrors(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        url = "%s%s" % (self.MOCKURL, config.METALINK_GOOD_01)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.fetchmirrors = True
        h.perform(r)

        self.assertEqual(h.mirrors, ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])
        self.assertEqual(h.metalink,
            {'timestamp': 1347459931,
             'hashes': [
                 ('md5', 'f76409f67a84bcd516131d5cc98e57e1'),
                 ('sha1', '75125e73304c21945257d9041a908d0d01d2ca16'),
                 ('sha256', 'bef5d33dc68f47adc7b31df448851b1e9e6bae27840f28700fff144881482a6a'),
                 ('sha512', 'e40060c747895562e945a68967a04d1279e4bd8507413681f83c322479aa564027fdf3962c2d875089bfcb9317d3a623465f390dc1f4acef294711168b807af0')],
             'size': 2621,
             'urls': [{
                 'url': 'http://127.0.0.1:%d/yum/static/01/repodata/repomd.xml' % self.PORT,
                 'type': 'http',
                 'protocol': 'http',
                 'location': 'CZ',
                 'preference': 100}],
             'filename': 'repomd.xml'}
            )

    def test_download_repo_01_with_baseurl_and_metalink_specified(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        url = "%s%s" % (self.MOCKURL, config.METALINK_GOOD_01)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.fastestmirror = True # XXX
        h.perform(r)

        self.assertEqual(h.mirrors, ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])
        self.assertEqual(h.metalink,
            {'timestamp': 1347459931,
             'hashes': [
                 ('md5', 'f76409f67a84bcd516131d5cc98e57e1'),
                 ('sha1', '75125e73304c21945257d9041a908d0d01d2ca16'),
                 ('sha256', 'bef5d33dc68f47adc7b31df448851b1e9e6bae27840f28700fff144881482a6a'),
                 ('sha512', 'e40060c747895562e945a68967a04d1279e4bd8507413681f83c322479aa564027fdf3962c2d875089bfcb9317d3a623465f390dc1f4acef294711168b807af0')],
             'size': 2621,
             'urls': [{
                 'url': 'http://127.0.0.1:%d/yum/static/01/repodata/repomd.xml' % self.PORT,
                 'type': 'http',
                 'protocol': 'http',
                 'location': 'CZ',
                 'preference': 100}],
             'filename': 'repomd.xml'}
            )

# Mirrorlist tests

    def test_download_only_mirrorlist(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_GOOD_01)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.fetchmirrors = True
        h.perform(r)

        self.assertEqual(h.mirrors, ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])
        self.assertEqual(h.metalink, None)

    def test_download_repo_01_via_mirrorlist_01(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_GOOD_01)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertEqual(yum_repo["url"], "http://127.0.0.1:%d/yum/static/01/" % self.PORT)
        self.assertTrue(yum_repomd)

    def test_download_repo_01_via_mirrorlist_02(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_GOOD_02)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertEqual(yum_repo["url"], "http://127.0.0.1:%d/yum/static/01/" % self.PORT)
        self.assertTrue(yum_repomd)

    def test_download_repo_01_via_mirrorlist_nourls(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_NOURLS)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertEqual(yum_repo, None)
        self.assertEqual(yum_repomd, None)

    def test_download_repo_01_via_mirrorlist_badfirsturl(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_BADFIRSTURL)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertEqual(yum_repo["url"], "http://127.0.0.1:%d/yum/static/01/" % self.PORT)

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

    def test_download_repo_01_via_mirrorlist_firsturlhascorruptedfiles(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_FIRSTURLHASCORRUPTEDFILES)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertEqual(yum_repo["url"],
            "http://127.0.0.1:%d/yum/harm_checksum/primary.xml/static/01/" % self.PORT)

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

    def test_download_repo_01_via_mirrorlist_firsturlhascorruptedfiles_maxmirrortries_enabled(self):
        """Download should fails on the first mirror (one file has a bad checksum).
        Other mirrors have the file with a good checksum, but option
        LRO_MAXMIRRORTRIES should prevent trying of other mirrors."""
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_FIRSTURLHASCORRUPTEDFILES)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.maxmirrortries = 1
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

    def test_download_repo_01_via_mirrorlist_firsturlhascorruptedfiles_maxmirrortries_enabled_2(self):
        """Download should fails on the first mirror (one file has a bad checksum).
        Other mirrors have the file with a good checksum.
        LRO_MAXMIRRORTRIES should allow try one next mirror. Thus repo
        should be downloaded without error."""
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_FIRSTURLHASCORRUPTEDFILES)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.maxmirrortries = 2
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertEqual(yum_repo["url"],
            "http://127.0.0.1:%d/yum/harm_checksum/primary.xml/static/01/" % self.PORT)

        # Test if all mentioned files really exist
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("url", "destdir")):
                self.assertTrue(os.path.isfile(yum_repo[key]))

    def test_download_repo_01_with_baseurl_and_mirrorlist_specified_only_fetchmirrors(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_GOOD_01)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.fetchmirrors = True
        h.perform(r)

        self.assertEqual(h.mirrors,
                         ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])
        self.assertEqual(h.metalink, None)

    def test_download_repo_01_with_baseurl_and_mirrorlist_specified(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_GOOD_01)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.perform(r)

        self.assertEqual(h.mirrors,
                         ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])
        self.assertEqual(h.metalink, None)

# Update test

    def test_download_and_update_repo_01(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_FIRSTURLHASCORRUPTEDFILES)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.yumdlist = [None]
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertEqual(yum_repo["url"],
            "http://127.0.0.1:%d/yum/harm_checksum/primary.xml/static/01/" % self.PORT)

        # Test that only repomd.xml has a path
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        self.assertTrue(os.path.exists(yum_repo["repomd"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("repomd", "url", "destdir", "mirrorlist")):
                self.assertTrue(yum_repo[key] == None)

        # Update repo
        h.update = True
        h.yumdlist = ["primary"]
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        # Test that only repomd.xml and primary has a path in yum_repo
        self.assertTrue(os.path.isdir(yum_repo["destdir"]))
        self.assertTrue(os.path.exists(yum_repo["repomd"]))
        self.assertTrue(os.path.exists(yum_repo["primary"]))
        for key in yum_repo:
            if yum_repo[key] and (key not in ("repomd", "primary", "url", "destdir", "mirrorlist")):
                self.assertTrue(yum_repo[key] == None)

# Base Auth test

    def test_download_repo_01_from_base_auth_secured_web_01(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s%s" % (self.MOCKURL, config.AUTHBASIC, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

    def test_download_repo_01_from_base_auth_secured_web_02(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s%s" % (self.MOCKURL, config.AUTHBASIC, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.httpauth = True
        h.userpwd = "%s:%s" % (config.AUTH_USER, config.AUTH_PASS)
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)

# Progressbar test

    def test_download_repo_01_with_progressbar(self):
        data = {"ttd": 0, "d": 0, "calls": 0}
        def cb(data, total_to_download, downloaded):
            data["ttd"] = total_to_download
            data["d"] = downloaded
            data["calls"] = data["calls"] + 1

        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.progressdata = data
        h.progresscb = cb
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertTrue(data["calls"] > 0)
        self.assertTrue(data["ttd"] == data["d"] or data["ttd"] == 0)

# Var substitution test

    def test_download_repo_01_with_url_substitution(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH_VAR)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.checksum = True
        h.varsub = config.REPO_YUM_01_VARSUB_LIST
        h.perform(r)

        yum_repo   = r.getinfo(librepo.LRR_YUM_REPO)
        yum_repomd = r.getinfo(librepo.LRR_YUM_REPOMD)

        self.assertTrue(yum_repo)
        self.assertTrue(yum_repomd)
        self.assertTrue(yum_repo["url"].endswith(config.REPO_YUM_01_PATH))

    def test_download_repo_01_mirrorlist_substitution(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_VARED)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.fetchmirrors = True
        h.varsub = config.MIRRORLIST_VARED_LIST
        h.perform(r)

        self.assertEqual(h.mirrors, ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])
        self.assertEqual(h.metalink, None)

    def test_download_repo_01_mirrorlist_with_url_substitution(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.MIRRORLIST_VARSUB)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.fetchmirrors = True
        h.varsub = config.MIRRORLIST_VARSUB_LIST
        h.perform(r)

        self.assertEqual(h.mirrors, ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])
        self.assertEqual(h.metalink, None)

    def test_download_repo_01_metalink_with_url_substitution(self):
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.METALINK_VARSUB)
        h.mirrorlist = url
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.fetchmirrors = True
        h.varsub = config.METALINK_VARSUB_LIST
        h.perform(r)

        self.assertEqual(h.mirrors, ['http://127.0.0.1:%d/yum/static/01/' % self.PORT])
        self.assertEqual(h.metalink["urls"],
            [{
                'url': 'http://127.0.0.1:%d/yum/static/$version/repodata/repomd.xml' % self.PORT,
                'type': 'http',
                'protocol': 'http',
                'location': 'CZ',
                'preference': 100}
            ])

    def test_download_with_gpgcheck_enabled_but_without_signature(self):
        # At first, download whole repository
        h = librepo.Handle()
        r = librepo.Result()

        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_02_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        h.gpgcheck = True
        self.assertRaises(librepo.LibrepoException, h.perform, (r))

    def test_download_with_custom_http_headers(self):
        h = librepo.Handle()
        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h.urls = [url]
        h.repotype = librepo.LR_YUMREPO
        h.destdir = self.tmpdir
        headers = ["Accept: audio/mpeg"]
        h.httpheader = headers
        del headers
        h.perform()

    def test_download_with_local_enabled(self):
        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)

        # Make local cache
        h_l = librepo.Handle()
        h_l.repotype = librepo.YUMREPO
        h_l.urls = [url]
        h_l.destdir = self.tmpdir
        h_l.perform()

        self.assertFalse(h_l.mirrors)  # No mirrors should be listed

        # Create a handle for the local cache
        h = librepo.Handle()
        h.repotype = librepo.YUMREPO
        h.urls = [self.tmpdir]
        h.metalinkurl = "%s%s" % (self.MOCKURL, config.METALINK_GOOD_01)
        h.local = True
        h.perform()

        self.assertFalse(h.mirrors)  # Remote metalik is specified
                                     # No mirrors should be listed

        # Disable local
        h.local = False

        # Try to download something (don't care that it doesn't exist)
        librepo.download_packages([librepo.PackageTarget("Foo", handle=h)])

        self.assertTrue(h.mirrors)  # List of mirrors should be re-initialized
                                    # and should contain mirrors from the metalink

    def test_download_with_offline_enabled_01(self):
        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        h = librepo.Handle()
        h.repotype = librepo.YUMREPO
        h.urls = [url]
        h.destdir = self.tmpdir
        h.offline = True
        # Only one remote URL is specified - this should fail
        self.assertRaises(librepo.LibrepoException, h.perform)

        shutil.rmtree(os.path.join(self.tmpdir, "repodata"))

        # But everything should be ok when we disable the option
        h.offline = False
        h.perform()

    def test_download_with_offline_enabled_02(self):
        url = "%s%s" % (self.MOCKURL, config.REPO_YUM_01_PATH)
        dir_01 = os.path.join(self.tmpdir, "01")
        dir_02 = os.path.join(self.tmpdir, "02")
        os.makedirs(dir_01)
        os.makedirs(dir_02)

        # Prepare local cache
        h_l = librepo.Handle()
        h_l.repotype = librepo.YUMREPO
        h_l.urls = [url]
        h_l.destdir = dir_01
        h_l.perform()

        # Let's work offline
        h = librepo.Handle()
        h.repotype = librepo.YUMREPO
        h.urls = ["http://foo.bar/xyz", dir_01]
        h.metalinkurl = "http://foo.bar/mtl"
        h.mirrorlisturl = "http://foo.bar/mrl"
        h.fastestmirror = True
        h.destdir = dir_02
        h.offline = True
        # Bad URLs for metalink and mirrorlist should be ok, they
        # should be ignored - we should work offline
        # First remote URL set in urls option should be ignored
        # Fastestmirror should be skipped
        h.perform()

    def test_download_with_offline_enabled_03(self):
        url_mtl = "%s%s" % (self.MOCKURL, config.METALINK_GOOD_01)
        url_mrl = "%s%s" % (self.MOCKURL, config.MIRRORLIST_GOOD_01)

        h = librepo.Handle()
        h.repotype = librepo.YUMREPO
        h.metalinkurl = url_mtl
        h.mirrorlisturl = url_mrl
        h.fastestmirror = True
        h.offline = True
        h.fetchmirrors = True
        h.destdir = self.tmpdir
        # We only want to fetch mirrors
        # But both metalink and mirrorlist is specified by remote URL
        # Output mirrorlist should be empty
        h.perform()
        self.assertFalse(h.mirrors)

        # But when we disable offline, it should work
        h.offline = False
        h.perform()
        self.assertTrue(h.mirrors)

    def test_download_with_offline_enabled_04(self):
        url_mtl = "%s%s" % (self.MOCKURL, config.METALINK_GOOD_01)
        url_mrl = "%s%s" % (self.MOCKURL, config.MIRRORLIST_GOOD_01)
        dir_01 = os.path.join(self.tmpdir, "01")
        dir_02 = os.path.join(self.tmpdir, "02")
        os.makedirs(dir_01)
        os.makedirs(dir_02)

        EXP_MRS = [u'http://127.0.0.1:%s/yum/static/01/' % self.PORT,
                   u'http://127.0.0.1:%s/yum/static/01/' % self.PORT]

        # 1) At first, work online
        h = librepo.Handle()
        h.repotype = librepo.YUMREPO
        h.metalinkurl = url_mtl
        h.mirrorlisturl = url_mrl
        h.fastestmirror = True
        h.offline = False
        h.destdir = dir_01
        h.perform()
        self.assertTrue(h.mirrors)

        # 2) Then switch to offline mode
        h.offline = True
        h.destdir = dir_02
        # We should not be able to download the repodata
        self.assertRaises(librepo.LibrepoException, h.perform)
        # Mirrors should still be there from the first try
        self.assertTrue(h.mirrors)
        self.assertEqual(h.mirrors, EXP_MRS)

        # 3) Let's stay offline but use repodata from first try
        shutil.rmtree(os.path.join(dir_02, "repodata"))
        h.urls = [dir_01]
        h.perform()
        # Mirrors should still be there from the first try
        self.assertTrue(h.mirrors)
        self.assertEqual(h.mirrors, EXP_MRS)

        # 4) Still offline, but mirrors get reset
        shutil.rmtree(os.path.join(dir_02, "repodata"))
        h.metalinkurl = url_mtl
        h.mirrorlisturl = url_mrl
        h.perform()
        # Mirrors should be reset and thus empty
        self.assertFalse(h.mirrors)

        # 5) Create a brand new handle and load a local mirrorlists
        shutil.rmtree(os.path.join(dir_02, "repodata"))
        h = librepo.Handle()
        h.repotype = librepo.YUMREPO
        h.metalinkurl = os.path.join(dir_01, "metalink.xml")
        h.mirrorlisturl = os.path.join(dir_01, "mirrorlist")
        h.fastestmirror = True
        h.offline = True
        h.destdir = dir_02
        # Perform should fail, because we are working offline
        self.assertRaises(librepo.LibrepoException, h.perform)
        # The local mirrorlists should be loaded
        self.assertTrue(h.mirrors)
        self.assertEqual(h.mirrors, EXP_MRS)





#!/usr/bin/env python

"""
librepo - download packages
"""

import librepo

if __name__ == "__main__":

    # Setup logging
    def debug_function(msg, _):
        print msg
    librepo.set_debug_log_handler(debug_function)

    # Prepare handle
    h = librepo.Handle()
    h.urls = ["http://beaker-project.org/yum/client-testing/Fedora21/"]
    h.repotype = librepo.YUMREPO

    # Prepare list of targets
    packages = []

    target = librepo.PackageTarget("beaker-19.1-1.fc19.src.rpm",
                                   handle=h,
                                   checksum_type=librepo.SHA256,
                                   checksum="3a07327702d15de2707518eb777c0837b2fc7c0cf4cb757331982f5ce6053867",
                                   resume=True)
    packages.append(target)

    target = librepo.PackageTarget("beaker-client-19.1-1.fc19.noarch.rpm",
                                   handle=h,
                                   expectedsize=381148)
    packages.append(target)

    target = librepo.PackageTarget("rhts-4.65-1.fc19.src.rpm", handle=h)
    packages.append(target)

    librepo.download_packages(packages)

    for target in packages:
        print "### %s: %s" % (target.local_path, target.err or "OK")
        print "Relative URL:      ", target.relative_url
        print "Destination:       ", target.dest
        print "Base URL:          ", target.base_url
        print "Checksum type:     ", target.checksum_type
        print "Expected checksum: ", target.checksum
        print "Resume:            ", bool(target.resume)
        print "Local path:        ", target.local_path
        print "Error:             ", target.err
        print




#!/usr/bin/python

"""
Example: Localize metadata files of a local repository

Use case:
We have a local repositository in the rpm-md format
and want to paths to all its metadata files.
Repomd content is just a bonus.
"""

import sys
import librepo
import pprint

METADATA_PATH = "downloaded_metadata"

if __name__ == "__main__":
    h = librepo.Handle()
    r = librepo.Result()
    # Repository with repodata in the rpm-md format
    h.setopt(librepo.LRO_REPOTYPE, librepo.LR_YUMREPO)
    # Path to metadata
    h.setopt(librepo.LRO_URLS, [METADATA_PATH])
    # Do not duplicate (copy) metadata, just localise them
    h.setopt(librepo.LRO_LOCAL, True)

    try:
        h.perform(r)
    except librepo.LibrepoException as e:
        rc, msg, general_msg = e
        print "Error: %s" % msg
        sys.exit(1)

    print "Repomd content:"
    pprint.pprint(r.getinfo(librepo.LRR_YUM_REPOMD))

    print "\nPaths to metadata files:"
    for data_type, path in r.getinfo(librepo.LRR_YUM_REPO).iteritems():
        print "%15s: %s" % (data_type, path)

    sys.exit(0)



#!/usr/bin/env python

"""
librepo - download a file
"""

import os
import sys
import shutil

import librepo

URL = "http://mirrors.fedoraproject.org/mirrorlist?repo=updates-released-f20&arch=x86_64"
DEST_FN_1 = "downloaded_mirrorlist_1.txt"
DEST_FN_2 = "downloaded_mirrorlist_2.txt"

def download_1():
    """Example of the simplest usage"""
    fd = os.open(DEST_FN_1, os.O_RDWR|os.O_CREAT|os.O_TRUNC, 0666)
    librepo.download_url(URL, fd)

def download_2():
    """Handle could be used if you need specify some network
    options like proxy server etc."""

    # NOTE: LRO_URLS and LRO_MIRRORLIST options of the handle are ignored!!

    h = librepo.Handle()
    #h.setopt(librepo.LRO_PROXY, "http://foo.proxy.bar:8080")

    fd = os.open(DEST_FN_2, os.O_RDWR|os.O_CREAT|os.O_TRUNC, 0666)
    librepo.download_url(URL, fd, handle=h)

if __name__ == "__main__":
    download_1()
    download_2()



import unittest
import os
import time
import librepo

from tests.base import TestCase, TEST_DATA

REPO_YUM_01_PATH = TEST_DATA+"/repo_yum_01/"

class TestCaseYum(unittest.TestCase):
    def test_yum_repomd_get_age(self):
        h = librepo.Handle()

        h.urls = [REPO_YUM_01_PATH]
        h.repotype = librepo.LR_YUMREPO
        h.local = True

        r = h.perform()

        repomd_path = r.getinfo(librepo.LRR_YUM_REPO)["repomd"]
        mtime = os.stat(repomd_path).st_mtime
        calculated_age = time.time() - mtime

        librepo_age =  librepo.yum_repomd_get_age(r)

        # Approximate comparison
        self.assertTrue(((calculated_age - 10) < librepo_age) or \
                         (calculated_age + 10) > librepo_age)



REPOS_YUM = "yum/static/"
HARMCHECKSUM = "yum/harm_checksum/%s/"
MISSINGFILE = "yum/not_found/%s/"
BADURL = "yum/badurl/"
BADGPG = "yum/badgpg/"
AUTHBASIC = "yum/auth_basic/"
PARTIAL = "yum/partial/"

AUTH_USER = "admin"
AUTH_PASS = "secret"

REPO_YUM_01_PATH = REPOS_YUM+"01/"
REPO_YUM_01_REPODATA = REPO_YUM_01_PATH+"repodata/"
REPO_YUM_01_REPOMD = REPO_YUM_01_REPODATA+"repomd.xml"
REPO_YUM_01_PATH_VAR = REPOS_YUM+"$version/"
REPO_YUM_01_VARSUB_LIST = [("version", "01")]

REPO_YUM_02_PATH = REPOS_YUM+"02/"

REPO_YUM_03_PATH = REPOS_YUM+"03/"

REPO_YUM_04_PATH = REPOS_YUM+"04/"

METALINK_DIR = "yum/static/metalink/"
METALINK_GOOD_01 = METALINK_DIR+"good_01.xml"
METALINK_GOOD_02 = METALINK_DIR+"good_02.xml"
METALINK_BADFILENAME = METALINK_DIR+"badfilename.xml"
METALINK_BADCHECKSUM = METALINK_DIR+"badchecksum.xml"
METALINK_NOURLS = METALINK_DIR+"nourls.xml"
METALINK_BADFIRSTURL = METALINK_DIR+"badfirsturl.xml"
METALINK_BADFIRSTHOST = METALINK_DIR+"badfirsthost.xml"
METALINK_FIRSTURLHASCORRUPTEDFILES = METALINK_DIR+"firsturlhascorruptedfiles.xml"
METALINK_VARSUB = METALINK_DIR+"varsub.xml"
METALINK_VARSUB_LIST = [("version", "01")]
METALINK_WITH_ALTERNATES = METALINK_DIR+"metalink_with_alternates.xml"
METALINK_MIRRORS_01 = METALINK_DIR+"mirrors_01.xml"

MIRRORLIST_DIR = "yum/static/mirrorlist/"
MIRRORLIST_GOOD_01 = MIRRORLIST_DIR+"good_01"
MIRRORLIST_GOOD_02 = MIRRORLIST_DIR+"good_02"
MIRRORLIST_NOURLS = MIRRORLIST_DIR+"nourls"
MIRRORLIST_BADFIRSTURL = MIRRORLIST_DIR+"badfirsturl"
MIRRORLIST_FIRSTURLHASCORRUPTEDFILES = MIRRORLIST_DIR+"firsturlhascorruptedfiles"
MIRRORLIST_VARED = MIRRORLIST_DIR+"$var"
MIRRORLIST_VARED_LIST = [("var", "varsub"), ("version", "01")]
MIRRORLIST_VARSUB = MIRRORLIST_DIR+"varsub"
MIRRORLIST_VARSUB_LIST = [("version", "01")]

# PACKAGE_<repo_id>_<package_id>
REPO_YUM_01_PACKAGES = REPO_YUM_01_PATH
PACKAGE_01_01 = "filesystem-2.4.44-1.fc16.i686.rpm"
PACKAGE_01_01_SHA256 = "3a2d4370e617056c9f76ec9ad1a7c699db8da90d3cc860943035ca0fda8136f4"

REPO_YUM_03_PACKAGES = REPO_YUM_03_PATH
PACKAGE_03_01 = "Archer-3.4.5-6.x86_64.rpm"

REPO_YUM_04_PACKAGES = REPO_YUM_04_PATH
PACKAGE_04_01 = "empty-0-0.src.rpm"



#!/usr/bin/env python

"""
librepo - download packages
"""

import os
import os.path
import time
import librepo

CACHE = "fastestmirror.cache"

LIBREPOPKG = "librepo-1.2.1-2.fc20.x86_64.rpm"
LAMEPKG = "lame-3.99.5-2.fc19.x86_64.rpm"

if __name__ == "__main__":

    # Setup logging
    def debug_function(msg, _):
        print msg
    #librepo.set_debug_log_handler(debug_function)

    # Remove packages if already exists
    def remove_pkg(filename):
        if os.path.exists(filename):
            os.remove(filename)
    remove_pkg(LIBREPOPKG)
    remove_pkg(LAMEPKG)

    # Prepare list of targets
    packages = []

    # Prepare first target
    h1 = librepo.Handle()
    h1.metalinkurl = "https://mirrors.fedoraproject.org/metalink?repo=fedora-20&arch=x86_64"
    h1.repotype = librepo.YUMREPO
    h1.fastestmirror = True
    h1.fastestmirrorcache = CACHE
    target = librepo.PackageTarget("Packages/l/"+LIBREPOPKG, handle=h1)
    packages.append(target)

    # Prepare second target
    h2 = librepo.Handle()
    h2.mirrorlisturl = "http://mirrors.rpmfusion.org/mirrorlist?repo=free-fedora-19&arch=x86_64"
    h2.repotype = librepo.YUMREPO
    h2.fastestmirror = True
    h2.fastestmirrorcache = CACHE
    target = librepo.PackageTarget(LAMEPKG, handle=h2)
    packages.append(target)

    t = time.time()
    librepo.download_packages(packages)
    print "Download duration: {0}s\n".format((time.time() - t))

    for target in packages:
        print "### %s: %s" % (target.local_path, target.err or "OK")
        print "Local path:        ", target.local_path
        print "Error:             ", target.err
        print




#!/usr/bin/python

"""
librepo - example of usage
"""

import os
import sys
import shutil
from pprint import pprint

import librepo

DESTDIR = "downloaded_metadata"
PROGRESSBAR_LEN = 50

def callback(data, total_to_download, downloaded):
    """Progress callback"""
    if total_to_download <= 0:
        return
    completed = int(downloaded / (total_to_download / PROGRESSBAR_LEN))
    print "[%s%s] %8s/%8s (%s)\r" % ('#'*completed, '-'*(PROGRESSBAR_LEN-completed), int(downloaded), int(total_to_download), data),
    sys.stdout.flush()

if __name__ == "__main__":
    # Prepare destination directory
    if os.path.exists(DESTDIR):
        if not os.path.isdir(DESTDIR):
            raise IOError("%s is not a directory" % DESTDIR)
        shutil.rmtree(DESTDIR)
    os.mkdir(DESTDIR)

    h = librepo.Handle() # Handle represents a download configuration
    r = librepo.Result() # Result represents an existing/downloaded repository

    # --- Mandatory arguments -------------------------------------------

    # URL of repository or URL of metalink/mirrorlist
    h.setopt(librepo.LRO_URLS, ["http://ftp.linux.ncsu.edu/pub/fedora/linux/releases/19/Everything/i386/os/"])
    #h.setopt(librepo.LRO_MIRRORLIST, "https://mirrors.fedoraproject.org/metalink?repo=fedora-source-17&arch=i386")
    # Note: LRO_URLS and LRO_MIRRORLIST could be set and used simultaneously
    #       and if download from LRO_URLS failed, then mirrorlist is used

    # Type of repository
    h.setopt(librepo.LRO_REPOTYPE, librepo.LR_YUMREPO)

    # --- Optional arguments --------------------------------------------

    # Make download interruptible
    h.setopt(librepo.LRO_INTERRUPTIBLE, True)

    # Destination directory for metadata
    h.setopt(librepo.LRO_DESTDIR, DESTDIR)

    # Check checksum of all files (if checksum is available in repomd.xml)
    h.setopt(librepo.LRO_CHECKSUM, True)

    # Callback to display progress of downloading
    h.setopt(librepo.LRO_PROGRESSCB, callback)

    # Set user data for the callback
    h.setopt(librepo.LRO_PROGRESSDATA, {'test': 'dict', 'foo': 'bar'})

    # Download only filelists.xml, prestodelta.xml
    # Note: repomd.xml is downloaded implicitly!
    # Note: If LRO_YUMDLIST is None -> all files are downloaded
    h.setopt(librepo.LRO_YUMDLIST, ["filelists", "prestodelta"])

    h.perform(r)

    # Get and show results
    pprint (r.getinfo(librepo.LRR_YUM_REPO))
    pprint (r.getinfo(librepo.LRR_YUM_REPOMD))

    # Whoops... I forget to download primary.xml.. Lets fix it!
    # Set LRO_UPDATE - only update existing Result
    h.setopt(librepo.LRO_UPDATE, True)
    h.setopt(librepo.LRO_YUMDLIST, ["primary"])
    h.perform(r)

    # List of mirrors
    # (In this case no mirrorlist is used -> list will contain only one url)
    # Example of access info via attr insted of .getinfo() method
    pprint (h.mirrors)

    # Get and show final results
    pprint (r.getinfo(librepo.LRR_YUM_REPO))
    pprint (r.getinfo(librepo.LRR_YUM_REPOMD))



import unittest
import librepo

def foo_cb(data, total_to_download, downloaded):
    pass

def foo_hmfcb(data, msg, url, metadata):
    pass

class TestCaseHandle(unittest.TestCase):

    def test_handle_exceptions(self):
        h = librepo.Handle()
        self.assertTrue(h)

        self.assertRaises(AttributeError, getattr, h, 'foobar_attr')
        self.assertRaises(AttributeError, setattr, h, 'foobar_attr', 'xyz')

        self.assertRaises(ValueError, h.getinfo, 999999)
        self.assertRaises(ValueError, h.setopt, 999999, 'xyz')

    def test_handle_setopt_getinfo(self):
        """No exception should be raised."""
        h = librepo.Handle()

        self.assertFalse(h.getinfo(librepo.LRI_UPDATE))
        h.setopt(librepo.LRO_UPDATE,  True)
        self.assertTrue(h.getinfo(librepo.LRI_UPDATE))
        h.setopt(librepo.LRO_UPDATE,  False)
        self.assertFalse(h.getinfo(librepo.LRI_UPDATE))
        h.setopt(librepo.LRO_UPDATE,  1)
        self.assertTrue(h.getinfo(librepo.LRI_UPDATE))
        h.setopt(librepo.LRO_UPDATE,  0)
        self.assertFalse(h.getinfo(librepo.LRI_UPDATE))

        self.assertEqual(h.getinfo(librepo.LRI_URLS), [])
        h.setopt(librepo.LRO_URLS, ["http://foo"])
        self.assertEqual(h.getinfo(librepo.LRI_URLS), ["http://foo"])
        h.setopt(librepo.LRO_URLS, [""])
        self.assertEqual(h.getinfo(librepo.LRI_URLS), [""])
        h.setopt(librepo.LRO_URLS, None)
        self.assertEqual(h.getinfo(librepo.LRI_URLS), [])

        self.assertEqual(h.getinfo(librepo.LRI_MIRRORLIST), None)
        h.setopt(librepo.LRO_MIRRORLIST, "http://ml")
        self.assertEqual(h.getinfo(librepo.LRI_MIRRORLIST), "http://ml")
        h.setopt(librepo.LRO_MIRRORLIST, "")
        self.assertEqual(h.getinfo(librepo.LRI_MIRRORLIST), "")
        h.setopt(librepo.LRO_MIRRORLIST, None)
        self.assertEqual(h.getinfo(librepo.LRI_MIRRORLIST), None)

        self.assertFalse(h.getinfo(librepo.LRI_LOCAL))
        h.setopt(librepo.LRO_LOCAL,  1)
        self.assertTrue(h.getinfo(librepo.LRI_LOCAL))
        h.setopt(librepo.LRO_LOCAL,  0)
        self.assertFalse(h.getinfo(librepo.LRI_LOCAL))

        self.assertFalse(h.getinfo(librepo.LRI_PROGRESSCB))
        h.setopt(librepo.LRO_PROGRESSCB, foo_cb)
        self.assertEqual(h.getinfo(librepo.LRI_PROGRESSCB), foo_cb)
        h.setopt(librepo.LRO_PROGRESSCB, None)
        self.assertFalse(h.getinfo(librepo.LRI_PROGRESSCB))

        data = {'a':'foo'}

        self.assertFalse(h.getinfo(librepo.LRI_PROGRESSDATA))
        h.setopt(librepo.LRO_PROGRESSDATA, data)
        self.assertEqual(h.getinfo(librepo.LRI_PROGRESSDATA), data)
        h.setopt(librepo.LRO_PROGRESSDATA, None)
        self.assertFalse(h.getinfo(librepo.LRI_PROGRESSDATA))

        self.assertEqual(h.getinfo(librepo.LRI_DESTDIR), None)
        h.setopt(librepo.LRO_DESTDIR,  "foodir")
        self.assertEqual(h.getinfo(librepo.LRI_DESTDIR), "foodir")
        h.setopt(librepo.LRO_DESTDIR,  None)
        self.assertEqual(h.getinfo(librepo.LRI_DESTDIR), None)
        h.setopt(librepo.LRO_DESTDIR,  "")
        self.assertEqual(h.getinfo(librepo.LRI_DESTDIR), "")

        self.assertEqual(h.getinfo(librepo.LRI_USERAGENT), None)
        h.setopt(librepo.LRO_USERAGENT,  "librepo/0.0")
        self.assertEqual(h.getinfo(librepo.LRI_USERAGENT), "librepo/0.0")
        h.setopt(librepo.LRO_USERAGENT,  None)
        self.assertEqual(h.getinfo(librepo.LRI_USERAGENT), None)
        h.setopt(librepo.LRO_USERAGENT,  "")
        self.assertEqual(h.getinfo(librepo.LRI_USERAGENT), "")

        self.assertEqual(h.getinfo(librepo.LRI_YUMDLIST), None)
        self.assertEqual(h.getinfo(librepo.LRI_RPMMDDLIST), None)
        h.setopt(librepo.LRO_YUMDLIST,  ["primary", "other"])
        self.assertEqual(h.getinfo(librepo.LRI_YUMDLIST), ["primary", "other"])
        self.assertEqual(h.getinfo(librepo.LRI_RPMMDDLIST), ["primary", "other"])
        h.setopt(librepo.LRO_YUMDLIST,  [])
        self.assertEqual(h.getinfo(librepo.LRI_YUMDLIST), [])
        self.assertEqual(h.getinfo(librepo.LRI_RPMMDDLIST), [])
        h.setopt(librepo.LRO_YUMDLIST,  [None])
        self.assertEqual(h.getinfo(librepo.LRI_YUMDLIST), [])
        self.assertEqual(h.getinfo(librepo.LRI_RPMMDDLIST), [])
        h.setopt(librepo.LRO_YUMDLIST,  None)
        self.assertEqual(h.getinfo(librepo.LRI_YUMDLIST), None)
        self.assertEqual(h.getinfo(librepo.LRI_RPMMDDLIST), None)

        h.setopt(librepo.LRO_RPMMDDLIST,  ["primary", "other"])
        self.assertEqual(h.getinfo(librepo.LRI_YUMDLIST), ["primary", "other"])
        self.assertEqual(h.getinfo(librepo.LRI_RPMMDDLIST), ["primary", "other"])

        self.assertEqual(h.getinfo(librepo.LRI_YUMBLIST), None)
        self.assertEqual(h.getinfo(librepo.LRI_RPMMDBLIST), None)
        h.setopt(librepo.LRO_YUMBLIST,  ["primary", "other"])
        self.assertEqual(h.getinfo(librepo.LRI_YUMBLIST), ["primary", "other"])
        self.assertEqual(h.getinfo(librepo.LRI_RPMMDBLIST), ["primary", "other"])
        h.setopt(librepo.LRO_YUMBLIST,  [])
        self.assertEqual(h.getinfo(librepo.LRI_YUMBLIST), [])
        self.assertEqual(h.getinfo(librepo.LRI_RPMMDBLIST), [])
        h.setopt(librepo.LRO_YUMBLIST,  [None])
        self.assertEqual(h.getinfo(librepo.LRI_YUMBLIST), [])
        self.assertEqual(h.getinfo(librepo.LRI_RPMMDBLIST), [])

        self.assertEqual(h.getinfo(librepo.LRI_MAXMIRRORTRIES), 0)
        h.setopt(librepo.LRO_MAXMIRRORTRIES, 1)
        self.assertEqual(h.getinfo(librepo.LRI_MAXMIRRORTRIES), 1)
        h.setopt(librepo.LRO_MAXMIRRORTRIES, None)
        self.assertEqual(h.getinfo(librepo.LRI_MAXMIRRORTRIES), 0)

        self.assertEqual(h.getinfo(librepo.LRI_VARSUB), None)
        h.setopt(librepo.LRO_VARSUB, [("bar", "foo")])
        self.assertEqual(h.getinfo(librepo.LRI_VARSUB), [("bar", "foo")])
        h.setopt(librepo.LRO_VARSUB, None)
        self.assertEqual(h.getinfo(librepo.LRI_VARSUB), None)

        self.assertEqual(h.getinfo(librepo.LRI_FASTESTMIRROR), False)
        h.setopt(librepo.LRO_FASTESTMIRROR, True)
        self.assertEqual(h.getinfo(librepo.LRI_FASTESTMIRROR), True)
        h.setopt(librepo.LRO_FASTESTMIRROR, False)
        self.assertEqual(h.getinfo(librepo.LRI_FASTESTMIRROR), False)

        self.assertEqual(h.getinfo(librepo.LRI_LOWSPEEDTIME), 30)
        h.setopt(librepo.LRO_LOWSPEEDTIME, 120)
        self.assertEqual(h.getinfo(librepo.LRI_LOWSPEEDTIME), 120)
        h.setopt(librepo.LRO_LOWSPEEDTIME, None)
        self.assertEqual(h.getinfo(librepo.LRI_LOWSPEEDTIME), 30)

        self.assertEqual(h.getinfo(librepo.LRI_LOWSPEEDLIMIT), 1000)
        h.setopt(librepo.LRO_LOWSPEEDLIMIT, 123)
        self.assertEqual(h.getinfo(librepo.LRI_LOWSPEEDLIMIT), 123)
        h.setopt(librepo.LRO_LOWSPEEDLIMIT, None)
        self.assertEqual(h.getinfo(librepo.LRI_LOWSPEEDLIMIT), 1000)

        self.assertFalse(h.getinfo(librepo.LRI_HMFCB))
        h.setopt(librepo.LRO_HMFCB, foo_hmfcb)
        self.assertEqual(h.getinfo(librepo.LRI_HMFCB), foo_hmfcb)
        h.setopt(librepo.LRO_HMFCB, None)
        self.assertFalse(h.getinfo(librepo.LRI_HMFCB))

        self.assertTrue(h.getinfo(librepo.LRI_SSLVERIFYPEER))
        h.setopt(librepo.LRO_SSLVERIFYPEER, 0)
        self.assertEqual(h.getinfo(librepo.LRI_SSLVERIFYPEER), False)
        h.setopt(librepo.LRO_SSLVERIFYPEER, None)
        self.assertTrue(h.getinfo(librepo.LRI_SSLVERIFYPEER))

        self.assertTrue(h.getinfo(librepo.LRI_SSLVERIFYHOST))
        h.setopt(librepo.LRO_SSLVERIFYHOST, 0)
        self.assertEqual(h.getinfo(librepo.LRI_SSLVERIFYHOST), False)
        h.setopt(librepo.LRO_SSLVERIFYHOST, None)
        self.assertTrue(h.getinfo(librepo.LRI_SSLVERIFYHOST))

        self.assertEqual(h.getinfo(librepo.LRI_SSLCLIENTCERT), None)
        h.setopt(librepo.LRO_SSLCLIENTCERT, "/etc/cert.pem")
        self.assertEqual(h.getinfo(librepo.LRI_SSLCLIENTCERT), "/etc/cert.pem")
        h.setopt(librepo.LRO_SSLCLIENTCERT, "")
        self.assertEqual(h.getinfo(librepo.LRI_SSLCLIENTCERT), "")
        h.setopt(librepo.LRO_SSLCLIENTCERT, None)
        self.assertEqual(h.getinfo(librepo.LRI_SSLCLIENTCERT), None)

        self.assertEqual(h.getinfo(librepo.LRI_SSLCLIENTKEY), None)
        h.setopt(librepo.LRO_SSLCLIENTKEY, "/etc/cert.key")
        self.assertEqual(h.getinfo(librepo.LRI_SSLCLIENTKEY), "/etc/cert.key")
        h.setopt(librepo.LRO_SSLCLIENTKEY, "")
        self.assertEqual(h.getinfo(librepo.LRI_SSLCLIENTKEY), "")
        h.setopt(librepo.LRO_SSLCLIENTKEY, None)
        self.assertEqual(h.getinfo(librepo.LRI_SSLCLIENTKEY), None)

        self.assertEqual(h.getinfo(librepo.LRI_SSLCACERT), None)
        h.setopt(librepo.LRO_SSLCACERT, "/etc/ca.pem")
        self.assertEqual(h.getinfo(librepo.LRI_SSLCACERT), "/etc/ca.pem")
        h.setopt(librepo.LRO_SSLCACERT, "")
        self.assertEqual(h.getinfo(librepo.LRI_SSLCACERT), "")
        h.setopt(librepo.LRO_SSLCACERT, None)
        self.assertEqual(h.getinfo(librepo.LRI_SSLCACERT), None)

        self.assertEqual(h.getinfo(librepo.LRI_IPRESOLVE), librepo.IPRESOLVE_WHATEVER)
        h.setopt(librepo.LRO_IPRESOLVE, librepo.IPRESOLVE_V6)
        self.assertEqual(h.getinfo(librepo.LRI_IPRESOLVE), librepo.IPRESOLVE_V6)
        h.setopt(librepo.LRO_IPRESOLVE, None)
        self.assertEqual(h.getinfo(librepo.LRI_IPRESOLVE), librepo.IPRESOLVE_WHATEVER)

        self.assertEqual(h.getinfo(librepo.LRI_ALLOWEDMIRRORFAILURES), 4)
        h.setopt(librepo.LRO_ALLOWEDMIRRORFAILURES, 1)
        self.assertEqual(h.getinfo(librepo.LRI_ALLOWEDMIRRORFAILURES), 1)
        h.setopt(librepo.LRO_ALLOWEDMIRRORFAILURES, None)

        self.assertEqual(h.getinfo(librepo.LRI_ADAPTIVEMIRRORSORTING), 1)
        h.setopt(librepo.LRO_ADAPTIVEMIRRORSORTING, 0)
        self.assertEqual(h.getinfo(librepo.LRI_ADAPTIVEMIRRORSORTING), 0)
        h.setopt(librepo.LRO_ADAPTIVEMIRRORSORTING, None)
        self.assertEqual(h.getinfo(librepo.LRI_ADAPTIVEMIRRORSORTING), 1)

        self.assertEqual(h.getinfo(librepo.LRI_GNUPGHOMEDIR), None)
        h.setopt(librepo.LRO_GNUPGHOMEDIR,  "/tmp/keyring/")
        self.assertEqual(h.getinfo(librepo.LRI_GNUPGHOMEDIR), "/tmp/keyring/")
        h.setopt(librepo.LRO_GNUPGHOMEDIR,  None)
        self.assertEqual(h.getinfo(librepo.LRI_GNUPGHOMEDIR), None)
        h.setopt(librepo.LRO_GNUPGHOMEDIR,  "")
        self.assertEqual(h.getinfo(librepo.LRI_GNUPGHOMEDIR), "")

        self.assertEqual(h.getinfo(librepo.LRI_FASTESTMIRRORTIMEOUT), 2.0)
        h.setopt(librepo.LRO_FASTESTMIRRORTIMEOUT,  32.256)
        self.assertEqual(h.getinfo(librepo.LRI_FASTESTMIRRORTIMEOUT), 32.256)
        h.setopt(librepo.LRO_FASTESTMIRRORTIMEOUT,  None)
        self.assertEqual(h.getinfo(librepo.LRI_FASTESTMIRRORTIMEOUT), 2.0)

        self.assertEqual(h.getinfo(librepo.LRI_HTTPHEADER), None)
        h.setopt(librepo.LRO_HTTPHEADER, ["Accept: text/xml", "charsets: utf-8"])
        self.assertEqual(h.getinfo(librepo.LRI_HTTPHEADER),
                         ["Accept: text/xml", "charsets: utf-8"])
        h.setopt(librepo.LRO_HTTPHEADER, None)
        self.assertEqual(h.getinfo(librepo.LRI_HTTPHEADER), None)

        self.assertEqual(h.getinfo(librepo.LRI_HTTPAUTHMETHODS), librepo.LR_AUTH_BASIC)
        h.setopt(librepo.LRO_HTTPAUTHMETHODS,  librepo.LR_AUTH_DIGEST)
        self.assertEqual(h.getinfo(librepo.LRI_HTTPAUTHMETHODS), librepo.LR_AUTH_DIGEST)
        h.setopt(librepo.LRO_HTTPAUTHMETHODS,  None)
        self.assertEqual(h.getinfo(librepo.LRI_HTTPAUTHMETHODS), librepo.LR_AUTH_BASIC)

        self.assertEqual(h.getinfo(librepo.LRI_PROXYAUTHMETHODS), librepo.LR_AUTH_BASIC)
        h.setopt(librepo.LRO_PROXYAUTHMETHODS,  librepo.LR_AUTH_DIGEST)
        self.assertEqual(h.getinfo(librepo.LRI_PROXYAUTHMETHODS), librepo.LR_AUTH_DIGEST)
        h.setopt(librepo.LRO_PROXYAUTHMETHODS,  None)
        self.assertEqual(h.getinfo(librepo.LRI_PROXYAUTHMETHODS), librepo.LR_AUTH_BASIC)

        self.assertEqual(h.getinfo(librepo.LRI_FTPUSEEPSV), 1)
        h.setopt(librepo.LRO_FTPUSEEPSV, 0)
        self.assertEqual(h.getinfo(librepo.LRI_FTPUSEEPSV), 0)
        h.setopt(librepo.LRO_FTPUSEEPSV, None)
        self.assertEqual(h.getinfo(librepo.LRI_FTPUSEEPSV), 1)


    def test_handle_setget_attr(self):
        """No exception should be raised."""
        h = librepo.Handle()

        self.assertFalse(h.update)
        h.update = True
        self.assertTrue(h.update)
        h.update = False
        self.assertFalse(h.update)
        h.update = 1
        self.assertTrue(h.update)
        h.update = 0
        self.assertFalse(h.update)

        self.assertEqual(h.urls, [])
        h.urls = "http://foo"
        self.assertEqual(h.urls, ["http://foo"])
        h.urls = ""
        self.assertEqual(h.urls, [""])
        h.urls = None
        self.assertEqual(h.urls, [])

        self.assertEqual(h.mirrorlist, None)
        h.mirrorlist = "http://ml"
        self.assertEqual(h.mirrorlist, "http://ml")
        h.mirrorlist = ""
        self.assertEqual(h.mirrorlist, "")
        h.mirrorlist = None
        self.assertEqual(h.mirrorlist, None)

        self.assertFalse(h.local)
        h.local =  1
        self.assertTrue(h.local)
        h.local =  0
        self.assertFalse(h.local)

        self.assertFalse(h.progresscb)
        h.progresscb = foo_cb
        self.assertEqual(h.progresscb, foo_cb)
        h.progresscb = None
        self.assertFalse(h.progresscb)

        data = {'a':'foo'}

        self.assertFalse(h.progressdata)
        h.progressdata = data
        self.assertEqual(h.progressdata, data)
        h.progressdata = None
        self.assertFalse(h.progressdata)

        self.assertEqual(h.destdir, None)
        h.destdir =  "foodir"
        self.assertEqual(h.destdir, "foodir")
        h.destdir =  None
        self.assertEqual(h.destdir, None)
        h.destdir =  ""
        self.assertEqual(h.destdir, "")

        self.assertEqual(h.useragent, None)
        h.useragent =  "librepo/0.0"
        self.assertEqual(h.useragent, "librepo/0.0")
        h.useragent =  None
        self.assertEqual(h.useragent, None)
        h.useragent =  ""
        self.assertEqual(h.useragent, "")

        self.assertEqual(h.yumdlist, None)
        h.yumdlist =  ["primary", "other"]
        self.assertEqual(h.yumdlist, ["primary", "other"])
        h.yumdlist =  []
        self.assertEqual(h.yumdlist, [])
        h.yumdlist =  [None]
        self.assertEqual(h.yumdlist, [])

        self.assertEqual(h.yumblist, None)
        h.yumblist =  ["primary", "other"]
        self.assertEqual(h.yumblist, ["primary", "other"])
        h.yumblist =  []
        self.assertEqual(h.yumblist, [])
        h.yumblist =  [None]
        self.assertEqual(h.yumblist, [])

        self.assertEqual(h.maxmirrortries, 0)
        h.maxmirrortries = 1
        self.assertEqual(h.maxmirrortries, 1)
        h.maxmirrortries = None
        self.assertEqual(h.maxmirrortries, 0)

        self.assertEqual(h.varsub, None)
        h.varsub = [("foo", "bar")]
        self.assertEqual(h.varsub, [("foo", "bar")])
        h.varsub = None
        self.assertEqual(h.varsub, None)

        self.assertEqual(h.fastestmirror, False)
        h.fastestmirror = True
        self.assertEqual(h.fastestmirror, True)
        h.fastestmirror = False
        self.assertEqual(h.fastestmirror, False)

        self.assertEqual(h.lowspeedtime, 30)
        h.lowspeedtime = 20
        self.assertEqual(h.lowspeedtime, 20)
        h.lowspeedtime = None
        self.assertEqual(h.lowspeedtime, 30)

        self.assertEqual(h.lowspeedlimit, 1000)
        h.lowspeedlimit = 55
        self.assertEqual(h.lowspeedlimit, 55)
        h.lowspeedlimit = None
        self.assertEqual(h.lowspeedlimit, 1000)

        self.assertFalse(h.hmfcb)
        h.hmfcb = foo_hmfcb
        self.assertEqual(h.hmfcb, foo_hmfcb)
        h.hmfcb = None
        self.assertFalse(h.hmfcb)

        self.assertTrue(h.sslverifypeer)
        h.sslverifypeer = False
        self.assertEqual(h.sslverifypeer, False)
        h.sslverifypeer = None
        self.assertTrue(h.sslverifypeer)

        self.assertTrue(h.sslverifyhost)
        h.sslverifyhost = False
        self.assertEqual(h.sslverifyhost, False)
        h.sslverifyhost = None
        self.assertTrue(h.sslverifyhost)

        self.assertEqual(h.ipresolve, librepo.IPRESOLVE_WHATEVER)
        h.ipresolve = librepo.IPRESOLVE_V4
        self.assertEqual(h.ipresolve, librepo.IPRESOLVE_V4)
        h.ipresolve = None
        self.assertEqual(h.ipresolve, librepo.IPRESOLVE_WHATEVER)

        self.assertEqual(h.allowedmirrorfailures, 4)
        h.allowedmirrorfailures = 1
        self.assertEqual(h.allowedmirrorfailures, 1)
        h.allowedmirrorfailures = None
        self.assertEqual(h.allowedmirrorfailures, 4)

        self.assertEqual(h.adaptivemirrorsorting, 1)
        h.adaptivemirrorsorting = 0
        self.assertEqual(h.adaptivemirrorsorting, 0)
        h.adaptivemirrorsorting = None
        self.assertEqual(h.adaptivemirrorsorting, 1)

        self.assertEqual(h.gnupghomedir, None)
        h.gnupghomedir =  "/tmp/keyring"
        self.assertEqual(h.gnupghomedir, "/tmp/keyring")
        h.gnupghomedir =  None
        self.assertEqual(h.gnupghomedir, None)
        h.gnupghomedir =  ""
        self.assertEqual(h.gnupghomedir, "")

        self.assertEqual(h.fastestmirrortimeout, 2.0)
        h.fastestmirrortimeout = 3.14
        self.assertEqual(h.fastestmirrortimeout, 3.14)
        h.fastestmirrortimeout = None
        self.assertEqual(h.fastestmirrortimeout, 2.0)

        self.assertEqual(h.httpheader, None)
        h.httpheader = ["Accept: text/xml", "charsets: utf-8"]
        self.assertEqual(h.httpheader, ["Accept: text/xml", "charsets: utf-8"])
        h.httpheader = None
        self.assertEqual(h.httpheader, None)

        self.assertEqual(h.httpauthmethods, librepo.LR_AUTH_BASIC)
        h.httpauthmethods = librepo.LR_AUTH_NTLM
        self.assertEqual(h.httpauthmethods, librepo.LR_AUTH_NTLM)
        h.httpauthmethods = None
        self.assertEqual(h.httpauthmethods, librepo.LR_AUTH_BASIC)

        self.assertEqual(h.proxyauthmethods, librepo.LR_AUTH_BASIC)
        h.proxyauthmethods = librepo.LR_AUTH_NTLM
        self.assertEqual(h.proxyauthmethods, librepo.LR_AUTH_NTLM)
        h.proxyauthmethods = None
        self.assertEqual(h.proxyauthmethods, librepo.LR_AUTH_BASIC)

        self.assertTrue(h.ftpuseepsv)
        h.ftpuseepsv = 0
        self.assertFalse(h.ftpuseepsv)
        h.ftpuseepsv = 1
        self.assertTrue(h.ftpuseepsv)
        h.ftpuseepsv = False
        self.assertFalse(h.ftpuseepsv)
        h.ftpuseepsv = True
        self.assertTrue(h.ftpuseepsv)
        h.ftpuseepsv = 0
        h.ftpuseepsv = None
        self.assertTrue(h.ftpuseepsv)


    def test_handle_setopt_none_value(self):
        """Using None in setopt."""
        h = librepo.Handle()

        h.setopt(librepo.LRO_LOCAL, None)
        h.local = None
        h.setopt(librepo.LRO_HTTPAUTH, None)
        h.httpauth = None
        h.setopt(librepo.LRO_USERPWD, None)
        h.userpwd = None
        h.setopt(librepo.LRO_PROXY, None)
        h.proxy = None
        h.setopt(librepo.LRO_PROXYPORT, None)       # None sets default value
        h.proxyport = None
        h.setopt(librepo.LRO_PROXYTYPE, None)
        h.proxytype = None
        h.setopt(librepo.LRO_PROXYAUTH, None)
        h.proxyauth = None
        h.setopt(librepo.LRO_PROXYUSERPWD, None)
        h.proxyuserpwd = None
        h.setopt(librepo.LRO_PROGRESSDATA, None)
        h.progressdata = None
        h.setopt(librepo.LRO_MAXSPEED, None)        # None sets default value
        h.maxspeed = None
        h.setopt(librepo.LRO_CONNECTTIMEOUT, None)  # None sets default value
        h.connecttimeout = None
        h.setopt(librepo.LRO_IGNOREMISSING, None)
        h.ignoremissing = None
        h.setopt(librepo.LRO_INTERRUPTIBLE, None)
        h.interruptible = None
        h.setopt(librepo.LRO_USERAGENT, None)
        h.useragent = None
        h.setopt(librepo.LRO_FETCHMIRRORS, None)
        h.fetchmirrors = None
        h.setopt(librepo.LRO_MAXMIRRORTRIES, None)
        h.maxmirrortries = None
        h.setopt(librepo.LRO_VARSUB, None)
        h.varsub = None
        h.setopt(librepo.LRO_FASTESTMIRROR, None)
        h.fastestmirror = None
        h.setopt(librepo.LRO_FASTESTMIRRORCACHE, None)
        h.fastestmirrorcache = None
        h.setopt(librepo.LRO_FASTESTMIRRORMAXAGE, None)
        h.fastestmirrormaxage = None
        h.setopt(librepo.LRO_FASTESTMIRRORDATA, None)
        h.fastestmirrordata = None
        h.setopt(librepo.LRO_LOWSPEEDTIME, None)
        h.lowspeedtime = None
        h.setopt(librepo.LRO_LOWSPEEDLIMIT, None)
        h.lowspeedlimit = None
        h.setopt(librepo.LRO_GPGCHECK, None)
        h.gpgcheck = None
        h.setopt(librepo.LRO_CHECKSUM, None)
        h.checksum = None
        h.setopt(librepo.LRO_LOWSPEEDTIME, None)
        h.lowspeedtime = None
        h.setopt(librepo.LRO_LOWSPEEDLIMIT, None)
        h.lowspeedlimit = None

        h.setopt(librepo.LRO_PROGRESSCB, None)
        h.progresscb = None
        h.setopt(librepo.LRO_PROGRESSCB, foo_cb)
        h.progresscb = foo_cb
        h.setopt(librepo.LRO_PROGRESSCB, None)
        h.progresscb = None

        def fmcallback(userdata, stage, data):
            pass
        h.setopt(librepo.LRO_FASTESTMIRRORCB, None)
        h.fastestmirrorcb = None
        h.setopt(librepo.LRO_FASTESTMIRRORCB, fmcallback)
        h.fastestmirrorcb = fmcallback
        h.setopt(librepo.LRO_FASTESTMIRRORCB, None)
        h.fastestmirror = None

        h.setopt(librepo.LRO_HMFCB, None)
        h.hmfcb = None
        h.setopt(librepo.LRO_HMFCB, foo_hmfcb)
        h.hmfcb = foo_hmfcb
        h.setopt(librepo.LRO_HMFCB, None)
        h.hmfcb = None

        h.setopt(librepo.LRO_SSLVERIFYPEER, None)
        h.sslverifypeer = None
        h.setopt(librepo.LRO_SSLVERIFYHOST, None)
        h.sslverifyhost = None
        h.setopt(librepo.LRO_SSLCLIENTCERT, None)
        h.sslclientcert = None
        h.setopt(librepo.LRO_SSLCLIENTKEY, None)
        h.sslclientkey = None
        h.setopt(librepo.LRO_SSLCACERT, None)
        h.sslcacert = None

        h.setopt(librepo.LRO_IPRESOLVE, None)
        h.ipresolve = None
        h.setopt(librepo.LRO_ALLOWEDMIRRORFAILURES, None)
        h.allowedmirrorfailures = None
        h.setopt(librepo.LRO_ADAPTIVEMIRRORSORTING, None)
        h.adaptivemirrorsorting = None

        h.setopt(librepo.LRO_GNUPGHOMEDIR, None)
        h.gnupghomedir = None
        h.setopt(librepo.LRO_FASTESTMIRRORTIMEOUT, None)
        h.fastestmirrortimeout = None
        h.setopt(librepo.LRO_HTTPHEADER, None)
        h.httpheader = None
        h.setopt(librepo.LRO_FTPUSEEPSV, None)
        h.ftpuseepsv = None

        h.httpauthmethods = None
        h.proxyauthmethods = None



#!/usr/bin/env python

"""
librepo - download packages from repo with fastestmirror and cache enabled
"""

import os
import os.path
import time
import librepo

METALINKURL = "https://mirrors.fedoraproject.org/metalink?repo=fedora-19&arch=x86_64"
PACKAGEPATH = "Packages/l/"
PACKAGE = "librepo-0.0.4-1.fc19.i686.rpm"
CACHE = "fastestmirror.cache"

def fastestmirrorstatuscallback(userdata, stage, data):
    if stage == librepo.FMSTAGE_INIT:
        print "Fastest mirror detection started.."
    elif stage == librepo.FMSTAGE_CACHELOADING:
        print "Fastest mirror cache used: {0}".format(data)
    elif stage == librepo.FMSTAGE_CACHELOADINGSTATUS:
        if data:
            print "Error while cache loading: {0}".format(data)
        else:
            print "Cache loaded successfully"
    elif stage == librepo.FMSTAGE_DETECTION:
        print "Fastest mirror detection in progress.. (%s mirror(s) will be pinged)" % data
    elif stage == librepo.FMSTAGE_FINISHING:
        print "Fastest mirror detection finishing.."
    elif stage == librepo.FMSTAGE_STATUS:
        if data:
            print "Fastest mirror detection error: {0}".format(data)
        else:
            print "Fastest mirror detection successful"
    else:
        print "UNKNOWN STAGE of Fastest mirror detection"

if __name__ == "__main__":

    # Setup logging
    def debug_function(msg, _):
        print msg
    #librepo.set_debug_log_handler(debug_function)

    def remove_cache():
        if os.path.exists(CACHE):
            print "Removing %s" % CACHE
            os.remove(CACHE)

    def remove_package():
        if os.path.exists(PACKAGE):
            print "Removing %s" % PACKAGE
            os.remove(PACKAGE)

    remove_cache()
    remove_package()

    print "\n1st run: Basic download"
    h = librepo.Handle()
    h.interruptible = True
    h.metalinkurl = METALINKURL
    h.repotype = librepo.YUMREPO
    packages = [librepo.PackageTarget(PACKAGEPATH+PACKAGE, handle=h)]
    t = time.time()
    librepo.download_packages(packages)
    t1 = time.time() - t
    print "### %s: %s (Time: %s)" % (
            packages[0].local_path, packages[0].err or "OK", t1)

    remove_package()

    print "\n2nd run: Download with fastestmirror - cache will be builded"
    h = librepo.Handle()
    h.interruptible = True
    h.metalinkurl = METALINKURL
    h.repotype = librepo.YUMREPO
    h.fastestmirror = True
    h.fastestmirrorcache = CACHE
    h.fastestmirrorcb = fastestmirrorstatuscallback
    packages = [librepo.PackageTarget(PACKAGEPATH+PACKAGE, handle=h)]
    t = time.time()
    librepo.download_packages(packages)
    t2 = time.time() - t
    print "### %s: %s (Time: %s)" % (
            packages[0].local_path, packages[0].err or "OK", t2)

    remove_package()

    print "\n3th run: Download with fastestmirror - cache should be used"
    h = librepo.Handle()
    h.interruptible = True
    h.metalinkurl = METALINKURL
    h.repotype = librepo.YUMREPO
    h.fastestmirror = True
    h.fastestmirrorcache = CACHE
    h.fastestmirrorcb = fastestmirrorstatuscallback
    packages = [librepo.PackageTarget(PACKAGEPATH+PACKAGE, handle=h)]
    t = time.time()
    librepo.download_packages(packages)
    t3 = time.time() - t
    print "### %s: %s (Time: %s)" % (
            packages[0].local_path, packages[0].err or "OK", t3)

    remove_package()
    remove_cache()



#!/usr/bin/python

"""
Example: Simple download whole repository

This example uses more "pythonic" way of usage.
Instead of use setopt() method it uses class properties.

Use case:
We have a metalink url of a repository and we
want do download complete repository metadata.
"""

import librepo

# Metalink URL
METALINK_URL = "https://mirrors.fedoraproject.org/metalink?repo=fedora-19&arch=x86_64"

# Destination directory (note: This directory must exists!)
DESTDIR = "downloaded_metadata"

if __name__ == "__main__":
    h = librepo.Handle()
    r = librepo.Result()
    # Repository with repodata in the rpm-md format
    h.repotype = librepo.LR_YUMREPO
    # Set metalink url
    h.mirrorlist = METALINK_URL
    # Destination directory for metadata
    h.destdir = DESTDIR
    # Use the fastest mirror
    h.fastestmirror = True

    try:
        h.perform(r)
    except librepo.LibrepoException as e:
        # rc - Return code (integer value)
        # msg - Detailed error message (string)
        # general_msg - Error message based on rc (string)
        rc, msg, general_msg  = e
        print "Error: %s" % msg



#!/usr/bin/env python

"""
librepo - download a package using client SSL certificate
"""
import librepo


if __name__ == "__main__":

    h = librepo.Handle()
    h.setopt(librepo.LRO_REPOTYPE, librepo.LR_YUMREPO)
    h.setopt(librepo.LRO_URLS, ["https://my-ssl-repo/fedora/20/x86_64/os/"])

    # The client certificate .pem file may include just the public certificate
    # or may include an entire certificate chain including
    # public key, private key, and root certificates.
    h.setopt(librepo.LRO_SSLCLIENTCERT, "/etc/pki/client.pem")

    # In case the .pem just contains the public key, set the private key
    h.setopt(librepo.LRO_SSLCLIENTKEY, "/etc/client.key")

    # Set the certificate authority
    h.setopt(librepo.LRO_SSLCACERT, "/etc/ca.pem")

    h.progressdata = "i2c-tools-eepromer"
    h.download("packages/i2c-tools-eepromer-24-1.x86_64.rpm")



import unittest
import librepo

class TestCaseVersion(unittest.TestCase):
    def test_version(self):
        self.assertTrue(len(librepo.VERSION) >= 5)
        self.assertEqual(librepo.VERSION.count('.'), 2)



#!/usr/bin/python

"""
Example: Verify checksum of local metadata

Use case:
We have some incomplete metadata localy.
They are incomplete because they doesn't
contain all files specified in repomd.xml.
They contains only primary.xml and filelists.xml.
We want to check checksum of this metadata.
"""

import sys
import librepo

METADATA_PATH = "downloaded_metadata"

if __name__ == "__main__":
    h = librepo.Handle()
    r = librepo.Result()
    # Repository with repodata in the rpm-md format
    h.setopt(librepo.LRO_REPOTYPE, librepo.LR_YUMREPO)
    # Path to the metadata
    h.setopt(librepo.LRO_URLS, [METADATA_PATH])
    # Do not duplicate (copy) the metadata
    h.setopt(librepo.LRO_LOCAL, True)
    # Check checksum of metadata
    h.setopt(librepo.LRO_CHECKSUM, True)
    # Ignore missing metadata files
    h.setopt(librepo.LRO_IGNOREMISSING, True)

    try:
        h.perform(r)
    except librepo.LibrepoException as e:
        rc, msg, general_msg = e
        if rc == librepo.LRE_BADCHECKSUM:
            print "Corrupted metadata! (%s)" % msg
        else:
            print "Other error: %s" % msg
        sys.exit(1)

    print "Metadata are fine!"
